/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Master task of the ddstest parallel routine.                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 04.04.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9                                                               */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <ppc/csp.h>
#include <ppc/parlib.h>

/*-------------------------------------------------------------------------*/
/* ppr_main:                                                               */
/* Master task entry point:                                                */
/*-------------------------------------------------------------------------*/
int ppr_main (
  channel from_host,
  channel to_host,
  channel from_slaves[],
  channel to_slaves[],
  int nb_row,
  int nb_col,
  char *matrix_name,
  int m,
  int n
)
{
  int c;
  int i;
  int j;
  double sum;
  double slave_result;
  double **matrix = (double **)dds_object_ptr(matrix_name);
  
  dds_send_name(matrix_name);

  for (c = 0; c < nb_row*nb_col; c++)
    send_channel(to_slaves[c],
      INT, m,
      INT, n,
      END);
      
  for (sum = 0.0, i = 0; i < m; i++)
    for (j = 0; j < n; j++)
      sum += matrix[i][j];

  for (c = 0; c < nb_row*nb_col; c++)
  {
    receive_channel(from_slaves[c],
      DOUBLE, &slave_result,
      END);
    if (slave_result != sum)
    {
      fprintf(stderr, "wrong result returned by slave %d\n", c);
      return -1;
    }
  }

  return PPR_OK;
}
