/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library.                                              */
/* nfact master.                                                           */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 01.04.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9                                                               */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <ppc/csp.h>
#include <ppc/parlib.h>

/*-------------------------------------------------------------------------*/
/* ppr_main:                                                               */
/* Routine master entry point.                                             */
/*-------------------------------------------------------------------------*/
int ppr_main (
  channel from_host,
  channel to_host,
  channel from_slaves[],
  channel to_slaves[],
  int nb_row,
  int nb_col,
  unsigned long lower,
  unsigned long upper,
  unsigned long *winner,
  unsigned long *count
)
{
  int numbers_per_slave;         /* # of numbers to analyse per slave task */
  int act_slave;                                           /* actual slave */
  unsigned long act_upper;
  unsigned long act_lower;
  unsigned long act_winner;
  unsigned long act_count;
  int nb_slaves = nb_row * nb_col;
  
  numbers_per_slave = (upper-lower)/nb_slaves;
  for (act_slave = 0; act_slave < nb_slaves; act_slave++)
  {
    act_lower = lower+(numbers_per_slave*act_slave);
    act_upper = act_lower+numbers_per_slave;
    if (act_slave == nb_slaves)
      act_upper = upper;

    send_channel(to_slaves[act_slave], 
      UNSIGNED_LONG, act_lower,
      UNSIGNED_LONG, act_upper,
      END);
  }
  
  *count = 0;
  for (act_slave = 0; act_slave < nb_slaves; act_slave++)
  {
    receive_channel(from_slaves[act_slave],
      UNSIGNED_LONG, &act_winner,
      UNSIGNED_LONG, &act_count,
      END);
    
    if (act_count > *count)
    {
      *count = act_count;
      *winner = act_winner;
    }
  }

  return PPR_OK;
}

