/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library.                                              */
/* nfact slave.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 01.04.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   PPC 0.9                                                               */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <ppc/csp.h>

/*-------------------------------------------------------------------------*/
/* calculate:                                                              */
/* Calculates the winner and its count of numbers between the given range. */
/*-------------------------------------------------------------------------*/
void calculate (
  unsigned long lower,                                      /* lower limit */
  unsigned long upper,                                      /* upper limit */
  unsigned long *winner,                      /* winner number set on exit */
  unsigned long *count                                    /* and its count */
)
{
  unsigned long act_number;               /* actual number being processed */
  unsigned long act_count;       /* number of factors of the actual number */
  
  act_number = lower;
  *count = 0;
  while(act_number <= upper)
  {
    act_count = calc_nfact(act_number);
    if (act_count > *count)
    {
      *count = act_count;
      *winner = act_number;
    }
    act_number++;
  }
}

/*-------------------------------------------------------------------------*/
/* ppr_main:                                                               */
/* Routine slave entry point.                                              */
/*-------------------------------------------------------------------------*/
void ppr_main (
  channel from_master,
  channel to_master,
  int nb_slaves,
  int my_nb
)
{
  unsigned long lower;
  unsigned long upper;
  unsigned long winner;
  unsigned long count;
  
  receive_channel(from_master,                           /* get my request */
    UNSIGNED_LONG, &lower,
    UNSIGNED_LONG, &upper, 
    END);
  
  calculate(lower, upper, &winner, &count);        /* determine the winner */
    
  send_channel(to_master,
    UNSIGNED_LONG, winner,
    UNSIGNED_LONG, count,
    END);
}

