/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* CHOLESKY parallel library routine.                                      */
/* implementation for full definite positive matrices.                     */
/* Slave.                                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.03.1995  Chevalier   Created                                         */
/* 25.04.1995  Droux       Cleanup, math.h inclusion                       */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   0.9.6                                                                 */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <math.h>
#include <stddef.h>
#include <ppc/csp.h>
#include <ppc/matrixutil.h>

void ppr_main(
  channel from_master,
  channel to_master,
  int my_number
)
{ 
  int i, k, col_counter; 
  int N ;
  int my_number_of_lines ;
  int current_index, current_j ;
  double sum ;
  double *j_line ;
  int *my_lines ;
  double **my_matrix ;
 
  /* Receive datas from Master: */

  receive_channel (from_master, 
    INT, &N, 
    INT, &my_number_of_lines, 
    END) ;
  

 my_matrix = dmatalloc(my_number_of_lines, N) ;


 j_line = dvecalloc(N) ;
  my_lines = ivecalloc(N) ;

  col_counter = 0 ;
  do
  {
    receive_channel (from_master,
      INT, &current_index, 
      END) ;
    my_lines[col_counter] = current_index ;

    receive_channel (from_master,
      DOUBLE_ARRAY, current_index, my_matrix[col_counter],

      END) ;

    col_counter++ ;
  } while (col_counter < my_number_of_lines) ;

  /* End of Receiving datas. */
   
  do
  {
    receive_channel (from_master,
      INT, &current_j, 
      DOUBLE_ARRAY, N, j_line,
      END) ;

    for (i=1; i<= my_number_of_lines; i++) 
    {
      current_index = my_lines[i-1] ;
      if (current_index > current_j)
      {
        sum = 0.0 ;
	for (k = 1; k<= current_j-1; k++)
	  sum = sum + j_line[k-1] * my_matrix[i-1][k-1] ;

	my_matrix[i-1][current_j-1] =   
	  (my_matrix[i-1][current_j-1] - sum) / j_line[current_j-1] ;
      }
             
      send_channel (to_master, 
	INT, current_index,
	DOUBLE, my_matrix[i-1][current_j-1] , 
	END) ; 
    }    
  } while (current_j < N) ; 


 dvecfree(j_line) ;
  dmatfree(my_matrix, my_number_of_lines) ;

 ivecfree(my_lines) ;
} 
