/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* CHOLESKY parallel library routine.                                      */
/* implementation for BANDED definite positive matrices.                   */
/* Slave.                                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.03.1995  Chevalier   Created                                         */
/* 25.04.1995  Droux       Cleanup, math.h inclusion                       */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   0.9.6                                                                 */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <math.h>
#include <stddef.h>
#include <ppc/csp.h>
#include <ppc/matrixutil.h>

/*-------------------------------------------------------------------------*/
/* ppr_main:                                                              */
/* Slave task entry point.                                                 */
/*-------------------------------------------------------------------------*/
void ppr_main(
  channel from_master,
  channel to_master,
  int nb_slaves,
  int my_number
)
{ 
  int i, k, line_counter; 
  int N, bandwidth, nterms, nline ;
  int my_number_of_lines, local_line, changed ;
  int current_index, current_j ;
  double sum, value_to_send ;
  double *j_line ;
  int *my_lines ;
  double **my_matrix ;
 
  /****************************************************************/
  /*************** BEGIN of slave task ****************************/
  /****************************************************************/

   
  
  /*************** Receive datas from Master: *********************/

  receive_channel (from_master, 
    INT, &N, 
    INT, &my_number_of_lines, 
    INT, &bandwidth,
    END) ;
    

 my_matrix = dmatalloc(my_number_of_lines, bandwidth+1) ;


 j_line = dvecalloc(N) ;
  my_lines = ivecalloc(N) ;

  line_counter = 0 ;
  do
  {
    receive_channel (from_master,
      INT, &current_index, 
      END) ;

    my_lines[line_counter] = current_index ;

    receive_channel (from_master,
      DOUBLE_ARRAY, bandwidth+1, my_matrix[line_counter],

      END) ;

    line_counter++ ;
  } while (line_counter < my_number_of_lines) ;

  /*************** End of Receiving datas. ****************************/
   
  do
  { 
    receive_channel (from_master,
      INT, &current_j, 
      DOUBLE_ARRAY, bandwidth+1, j_line,
      END) ;

    for (i=1; i<=bandwidth;i++)
    {
      nterms = bandwidth-i ;
      nline = current_j + i ;	 
      local_line = -1 ;
      changed = -1 ;
      value_to_send = 0.0 ;
      for (line_counter=0; line_counter <= my_number_of_lines; 
           line_counter++)
        if (my_lines[line_counter] == nline)
	  local_line = line_counter;      

      if (local_line >= 0)
      {
	sum = 0.0 ;
	for (k=1; k<=nterms; k++) 
	  sum = sum + j_line[bandwidth-k] *
	    my_matrix[local_line][bandwidth-k-i] ;
      
	if (nline<=N)
	{
	  changed = 1;
	  my_matrix[local_line][bandwidth-i] = 
	    (my_matrix[local_line][bandwidth-i] - sum)/j_line[bandwidth];
	}
	value_to_send = my_matrix[local_line][bandwidth-i] ;
      }

      send_channel (to_master,
	INT, changed, 
	INT, nline,
	INT, bandwidth-i,
	DOUBLE, value_to_send, 
	END) ; 
    }
  } while (current_j < N);

  dvecfree(j_line) ;
  ivecfree(my_lines) ;
  dmatfree(my_matrix, my_number_of_lines) ;
}
