/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* LEAST-SQUARES CONJUGATE-GRADIENT parallel library routine.              */
/* implementation for sparse definite positive matrices.                   */
/* Slave.                                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.03.1995  Chevalier   Created                                         */
/* 25.04.1995  Droux       Cleanup, math.h inclusion                       */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   0.9.6                                                                 */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <math.h>
#include <stddef.h>
#include <ppc/csp.h>
#include <ppc/matrixutil.h>

void ppr_main (
  channel from_master,
  channel to_master,
  channel from_next,
  channel to_next,
  channel from_prev,
  channel to_prev,
  int nb_slaves,
  int my_number
)
{ 
  int i, length, N, terminate ;
  double *myvalslice, mydotp, dotp ;
  int *mylinslice, *mycolslice ;
  double *myvecP, *myvecW, *partialsum ;

  receive_channel (from_master, 
    INT, &length,
    INT, &N,
    END) ;

  myvalslice = dvecalloc(length) ;
  mylinslice = ivecalloc(length) ;
  mycolslice = ivecalloc(length) ;
  myvecP = dvecalloc(N) ;
  myvecW = dvecalloc(N) ;
  partialsum = dvecalloc(N) ;

  receive_channel (from_master, 
    DOUBLE_ARRAY, length, myvalslice, 
    INT_ARRAY, length, mylinslice, 
    INT_ARRAY, length, mycolslice, 
    END) ; 
 
  receive_channel (from_master, 
    DOUBLE_ARRAY, N, myvecP, 
    END) ; 

  mydotp = 0.0 ; 
  dveczero (partialsum, N) ;

  for(i=1; i<=length;i++)
  {
    partialsum[mylinslice[i-1]-1] = partialsum[mylinslice[i-1]-1] +  
      myvalslice[i-1]*myvecP[mycolslice[i-1]-1];
    mydotp = mydotp + 
      myvalslice[i-1]*myvecP[mycolslice[i-1]-1]*myvecP[mylinslice[i-1]-1] ;
  }

  if(my_number>0) 
    receive_channel (from_prev, 
      DOUBLE_ARRAY, N, myvecW, 
      DOUBLE, &dotp,
      END) ; 
  else
    receive_channel (from_master, 
      DOUBLE_ARRAY, N, myvecW, 
      DOUBLE, &dotp,
      END) ; 
  
  dotp = dotp + mydotp ;
  
  for(i=1; i<=N;i++)
    myvecW[i-1] = myvecW[i-1] + partialsum[i-1] ;
  
  if(my_number<nb_slaves-1) 
    send_channel (to_next, 
      DOUBLE_ARRAY, N, myvecW,
      DOUBLE, dotp, 
      END) ; 
  else
    send_channel (to_master, 
      DOUBLE_ARRAY, N, myvecW,
      DOUBLE, dotp, 
      END) ; 
   
  terminate = 0 ;
  do
  {
    receive_channel (from_master, 
      DOUBLE_ARRAY, N, myvecP, 
      END) ; 

    mydotp = 0.0 ; 
    dveczero (partialsum, N) ;

    for(i=1; i<=length;i++)
    {
      partialsum[mylinslice[i-1]-1] = partialsum[mylinslice[i-1]-1] +
        myvalslice[i-1]*myvecP[mycolslice[i-1]-1];
      mydotp = mydotp + 
      myvalslice[i-1]*myvecP[mycolslice[i-1]-1]*myvecP[mylinslice[i-1]-1] ;
    }

    if(my_number>0) 
      receive_channel (from_prev, 
	DOUBLE_ARRAY, N, myvecW, 
	DOUBLE, &dotp,
	END) ; 
    else
      receive_channel (from_master, 
	DOUBLE_ARRAY, N, myvecW, 
	DOUBLE, &dotp,
	END) ; 

    dotp = dotp + mydotp ;

    for(i=1; i<=N;i++)
      myvecW[i-1] = myvecW[i-1] + partialsum[i-1] ;

    if(my_number<nb_slaves-1)
      send_channel (to_next, 
	DOUBLE_ARRAY, N, myvecW,
	DOUBLE, dotp, 
	END) ; 
    else
      send_channel (to_master, 
	DOUBLE_ARRAY, N, myvecW,
	DOUBLE, dotp, 
	END) ; 

    receive_channel (from_master, 
      INT, &terminate,
      END) ;
  } while(terminate < 1) ;

  dvecfree(myvalslice) ;
  dvecfree(myvecP) ;
  dvecfree(myvecW) ;
  dvecfree(partialsum) ;
  ivecfree(mylinslice) ;
  ivecfree(mycolslice) ;
} 
