/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel Programming Language.                            */
/* Parallel implementation of Conjugate-Gradient algorithm for a sparse    */
/* symmetric and positive definite matrix.                                 */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------------------------------------------------------------------- */
/* 23.11.1994  Chevalier   Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: main.c                                                        */
/* Tested on :                                                             */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*-------------------------------------------------------------------------*/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ppc/parlib.h>
#include <ppc/matrixutil.h>
#include <parlib/numerical.h>
#include "def_sparse.h"

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                              */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/

void ppc_user_main (void)
{  int i, bs ;
   int N, nonZeroNumber ;
   int *linvector, *colvector ;
   time_t l1, l2, elapsed_time ;
   double *valvector ; 
   double *vecB, *vecX ;
   
       
 /***********************************************************/     
  
   printf("please enter block size : "); scanf("%d", &bs);
   printf ("\n") ;

   N = bs*bs ;

   printf("matrix size: %d  \n", N) ;


  valvector = dvecalloc(5*N-1) ;

  linvector = ivecalloc(5*N-1) ;
  
colvector = ivecalloc(5*N-1) ;


  vecB = dvecalloc(N) ;

  vecX = dvecalloc(N) ;

   printf ("Filling of sparse-matrix structures.\n") ;

   define_sparse_matrix1 (bs, valvector, linvector, 
		colvector, &nonZeroNumber) ; 


 /***********************************************************/     
		
   printf ("Filling of vector vecB.\n") ;
 
   for (i=1 ; i <= N ; i++)
     vecB[i-1] = 1.0 ;
 
/*  dvecprint (vecB, N, "%4.4f ") ;
   printf ("\n") ;     
*/

/***********************************************************/     
/***********************************************************/     
   time (&l1) ;

   conj_grad(nonZeroNumber, N, valvector, linvector, colvector, 
             vecB, vecX) ;

   time (&l2) ;
 
   elapsed_time = difftime (l2, l1) ;
   printf ("elapsed time : %d   --> %d mn %d sec \n", \
          elapsed_time,
	  elapsed_time / 60, \
          elapsed_time % 60) ;  


  /****************************************************************/ 
  /************* Results and Check of results ********************/ 

   printf ("Vector solution: \n") ;
   dvecprint (vecX, N, "%4.4f ") ;
   printf ("\n") ;  


/***********************************************************/     

   dvecfree(valvector) ;
   dvecfree(vecB) ;
   dvecfree(vecX) ;
   ivecfree(linvector) ;
   ivecfree(colvector) ;
 

   
  
   printf ("Finished. \n") ;     
   printf ("\n") ;     

  /******** END.*************************************************/
}
