/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Complex numbers test.                                                   */
/* Master task.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 01.05.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.7                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ppc/csp.h>

#define SIZE 100

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel to_host,
  channel from_host,
  channel to_slaves [],
  channel from_slaves [],
  int nb_slaves
)
{
  int n;
  int c;
  ppc_complex cv [SIZE];
  ppc_complex rv [SIZE];
  ppc_complex *factor;
  
  factor = malloc(nb_slaves*sizeof(ppc_complex));

  for (c = 0; c < SIZE; c++)
  {
    cv[c].r = (double)c;
    cv[c].i = -(double)c;
  }
  
  printf("master: sending data\n");
  
  for (n = 0; n < nb_slaves; n++)
  {
    factor[n].r = (double)n+1.0;
    factor[n].i = -((double)n+1.0);

    send_channel(to_slaves[n],
      INT, SIZE,
      END);

    send_channel(to_slaves[n], 
      COMPLEX_ARRAY, SIZE, cv,
      COMPLEX, factor[n],
      END);
  }
   
  printf("master: receiving and checking results\n");
  
  for (n = 0; n < nb_slaves; n++)
  {
    receive_channel(from_slaves[n], 
      COMPLEX_ARRAY, SIZE, rv,
      END);
      
    for (c = 0; c < SIZE; c++)
      if (rv[c].r != cv[c].r * factor[n].r || 
          rv[c].i != cv[c].i * factor[n].i)
      {
        printf("master: slave %d returned wrong result: ", n);
	printf("(%f,%f) != (%f,%f)\n", 
	  rv[c].r, rv[c].i,
	  cv[c].r * factor[n].r, cv[c].i * factor[n].i);
	exit(1);
      }
  }

  printf("master: results are correct\n");
}
