/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Paragon channels specific functions interface.                          */
/* The functions found in this files are implemented in chanparagon.c.     */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 06.01.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: chanparagon.h                                                 */
/* Tested on :                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanparagon_h
#define chanparagon_h

#include <ppc/csp/chan.h>

#define PARAGON_CHANNELS 10000                     /* max Paragon channels */

typedef enum     
{                           
  PGON_CHAN,                          /* channel between two Paragon tasks */
  CSLIB_CHAN             /* between the PPC Paragon program and the client */
} paragon_chan_types;

/*-------------------------------------------------------------------------*/
/* new_pgon_channel:                                                       */
/* Declares a new Paragon channel.                                         */
/*-------------------------------------------------------------------------*/
channel new_pgon_channel (long dest_node, long type);

/*-------------------------------------------------------------------------*/
/* new_cslib_channel:                                                      */
/* Return the two channels from and to the client application and          */
/* initialize them with the given values.                                  */
/* This function is called by the unixconn module.                         */
/*-------------------------------------------------------------------------*/
void new_cslib_channel(
  int from_client,
  int to_client,
  channel *from_client_chan,
  channel *to_client_chan
);

/*-------------------------------------------------------------------------*/
/* pgon_correspondent_node:                                                */
/* Return the correspondent node of the specified channel.                 */
/*-------------------------------------------------------------------------*/
int pgon_correspondent_node (channel chan);

#endif
