/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* DDS private functions.                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.03.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: ddspriv.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef DDSPRIV_H
#define DDSPRIV_H

#include <ppc/parlib/dds.h>

typedef struct _dds_object
{
  char name[DDS_MAX_NAME_LEN];
  types_names type;
  DDS_DISTRIBUTION distribution;
  int nb_dim;
  int dim[DDS_MAX_DIM];
  void *value;
  struct _dds_object *next;
  struct _dds_object *prev;
} dds_object;

typedef enum                                        /* DDS kernel requests */
{
  DDSI_REQ_CLEAR,                                    /* dds_clear_database */
  DDSI_REQ_CREATE,                                           /* dds_create */
  DDSI_REQ_DELETE,                                           /* dds_delete */
  DDSI_REQ_COPY,                                               /* dds_copy */
  DDSI_REQ_INIT,                                                /* dds_set */
  DDSI_REQ_STORE                                              /* dds_store */
} DDSI_REQ;

/*-------------------------------------------------------------------------*/
/* ddsi_master_init:                                                       */
/* Initialize the DDS. Called by the master.                               */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_init (void);

/*-------------------------------------------------------------------------*/
/* ddsi_master_terminate:                                                  */
/* Terminate the DDS. Called by the master.                                */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_terminate (void);

/*-------------------------------------------------------------------------*/
/* ddsi_master_send_object_prop:                                           */
/* Send the given object property to the slaves.                           */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_send_object_prop (
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_master_add_object:                                                 */
/* Add given object to master's objects list.                              */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_add_object (
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_master_clear_database:                                             */
/* Clear master's objects database.                                        */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_clear_database (void);

/*-------------------------------------------------------------------------*/
/* ddsi_master_remove_object:                                              */
/* Remove object from master's object list.                                */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_remove_object (
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_master_get_first_object:                                           */
/* Return the first object in the objects list.                            */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
dds_object *ddsi_master_get_first_object (void);

/*-------------------------------------------------------------------------*/
/* ddsi_master_get_first_object:                                           */
/* Return the first object in the objects list.                            */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
dds_object *ddsi_master_get_first_object (void);

/*-------------------------------------------------------------------------*/
/* ddsi_master_send_req:                                                   */
/* Send a DDS request to the slaves.                                       */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_send_req (
  DDSI_REQ req
);

/*-------------------------------------------------------------------------*/
/* ddsi_slave_init:                                                        */
/* Initialize the DDS. Called by the slave.                                */
/* ddsslave.c                                                              */
/*-------------------------------------------------------------------------*/
void ddsi_slave_init (void);

/*-------------------------------------------------------------------------*/
/* ddsi_slave_terminate:                                                   */
/* Terminate the DDS. Called by the slave.                                 */
/* ddsslave.c                                                              */
/*-------------------------------------------------------------------------*/
void ddsi_slave_terminate (void);

/*-------------------------------------------------------------------------*/
/* ddsi_slave_handler:                                                     */
/* The parallel library kernel received a DDS request.                     */
/* ddsslave.c                                                              */
/*-------------------------------------------------------------------------*/
void ddsi_slave_handler (void);

/*-------------------------------------------------------------------------*/
/* ddsi_slave_receive_object_prop:                                         */
/* Receive an object property from the master.                             */
/* Allocates the object property fields if necessary.                      */
/* ddsslave.c                                                              */
/*-------------------------------------------------------------------------*/
void ddsi_slave_receive_object_prop (
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_slave_get_first_object:                                            */
/* Return the first object in the slave's objects list.                    */
/* ddsslave.c                                                              */
/*-------------------------------------------------------------------------*/
dds_object *ddsi_slave_get_first_object (void);

#define DDSI_GET_ACK { \
  int ack, ackC; \
  for (ackC = 0; ackC < nb_slaves; ackC++) \
    receive_channel(from_slaves[ackC], INT, &ack, END); }

#define DDSI_SEND_ACK send_channel(to_master, INT, 0, END);

#endif
