/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel Library manager kernel definitions for master.                 */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.01.1995  Droux       Created                                         */
/* 28.03.1995  Droux       Define channel type only if not included by DDS */
/* 29.03.1995  Droux       Added mkern_dds_req()                           */
/*-------------------------------------------------------------------------*/
/* filename: mkernel.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef PPC_LIB_MKERNEL_H
#define PPC_LIB_MKERNEL_H

#ifndef DDS_MASTER_IMPL
#ifndef MKERNEL_IMPL
#include <ppc/csp.h>
#else
typedef int channel;
#define CHANNEL_TYPE_DEFINED
#endif
#endif

#if !defined(MKERNEL_IMPL) && !defined(MKERNEL_CALLER) || defined(DDS_IMPL)
extern channel mkern_to_host;
extern channel mkern_from_host;
extern channel *mkern_to_slaves;
extern channel *mkern_from_slaves;
extern int mkern_nb_slaves;
extern int mkern_nb_row;
extern int mkern_nb_col;
#endif

/*-------------------------------------------------------------------------*/
/* mkern_up:                                                               */
/* Initializes the library master kernel.                                  */
/* Called by the master starter generated by the PPC configurer.           */
/*-------------------------------------------------------------------------*/
void mkern_up (
  channel to_host,
  channel from_host,
  channel to_slaves[],
  channel from_slaves[],
  int nb_row,
  int nb_col
);

/*-------------------------------------------------------------------------*/
/* mkern_start_routine:                                                    */
/* Starts the specified routine on the slaves.                             */
/*-------------------------------------------------------------------------*/
void mkern_start_routine (
  char *routine_name
);

/*-------------------------------------------------------------------------*/
/* mkern_down:                                                             */
/* Shutdown the master kernel.                                             */
/*-------------------------------------------------------------------------*/
void mkern_down (void);

/*-------------------------------------------------------------------------*/
/* mkern_dds_req:                                                          */
/* Send DDS request to slave kernels.                                      */
/*-------------------------------------------------------------------------*/
void mkern_dds_req (void);

/*-------------------------------------------------------------------------*/
/* mkern_dds_req_to:                                                       */
/* Send DDS request to a specific slave kernel.                            */
/*-------------------------------------------------------------------------*/
void mkern_dds_req_to (
  int slave_id
);

#if !defined(MKERNEL_IMPL) && !defined(MKERNEL_CALLER) || defined(DDS_IMPL)
#define to_host 	mkern_to_host
#define from_host 	mkern_from_host
#define to_slaves 	mkern_to_slaves
#define from_slaves 	mkern_from_slaves
#define nb_slaves 	mkern_nb_slaves
#define nb_row 		mkern_nb_row
#define nb_col 		mkern_nb_col
#endif

#endif
