/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel Library manager kernel definitions for slave.                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.01.1995  Droux       Created                                         */
/* 08.03.1995  Droux       skern_up() not declared when compiling skernel  */
/* 28.03.1995  Droux       Define channel type only if not included by DDS */
/*-------------------------------------------------------------------------*/
/* filename: skernel.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef PPC_LIB_SKERNEL_H
#define PPC_LIB_SKERNEL_H

#ifndef DDS_SLAVE_IMPL
#ifndef SKERNEL_IMPL
#include <ppc/csp.h>
#else
typedef int channel;
#endif
#endif

#if !defined(SKERNEL_IMPL) && !defined(SKERNEL_CALLER)
extern channel skern_to_master;
extern channel skern_from_master;
extern channel skern_to_next;                             
extern channel skern_from_next;
extern channel skern_to_prev;
extern channel skern_from_prev;
extern channel skern_to_north;                                
extern channel skern_from_north;
extern channel skern_to_south;
extern channel skern_from_south;
extern channel skern_to_west;
extern channel skern_from_west;
extern channel skern_to_east;
extern channel skern_from_east;
extern int skern_nb_slaves;                  
extern int skern_my_nb;
extern int skern_nb_row;                                        
extern int skern_nb_col;
extern int skern_my_row;
extern int skern_my_col;
#endif

/*-------------------------------------------------------------------------*/
/* skern_up:                                                               */
/* Initializes the library slave kernel.                                   */
/* Called by the slave starter generated by the PPC configurer.            */
/*-------------------------------------------------------------------------*/
#ifndef SKERNEL_IMPL
void skern_up (
  channel to_master,
  ...
);
#endif

/*-------------------------------------------------------------------------*/
/* skern_sleeper:                                                          */
/* Responds to master requests.                                            */
/*-------------------------------------------------------------------------*/
void skern_sleeper (void);

/*-------------------------------------------------------------------------*/
/* skern_down:                                                             */
/* Shutdown the slave kernel.                                              */
/*-------------------------------------------------------------------------*/
void skern_down (void);

#if !defined(SKERNEL_IMPL) && !defined(SKERNEL_CALLER)
#define to_master 	skern_to_master                                
#define from_master 	skern_from_master
#define to_next		skern_to_next                                  
#define from_next	skern_from_next
#define to_prev		skern_to_prev
#define from_prev	skern_from_prev
#define to_north	skern_to_north                                  
#define from_north	skern_from_north
#define to_south	skern_to_south
#define from_south	skern_from_south
#define to_west		skern_to_west
#define from_west	skern_from_west
#define to_east		skern_to_east
#define from_east	skern_from_east
#define nb_slaves	skern_nb_slaves                          
#define my_nb		skern_my_nb
#define nb_row		skern_nb_row                                        
#define nb_col		skern_nb_col
#define my_row		skern_my_row
#define my_col		skern_my_col
#endif

#endif

