
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include    "linkio.h"                  /* link interface definition */
#include "b004next.h"

typedef int bool_t;

#define TRUE    1
#define FALSE   0

#define NULL_LINK -1

static link_t ActiveLink = NULL_LINK;

/*
 *   Open Link
 */

link_t  OpenLink (char *link)
{
   if ( ActiveLink != NULL_LINK )
   {
      return( ER_LINK_CANT );
   }
      
   if ( link == NULL)
   {
     if ( ( ActiveLink = open( "/dev/b004" , O_RDWR ) ) < 0 )
     {
        return( ER_LINK_CANT );
     }
   }
   else
   {
     if ( ( ActiveLink = open( link , O_RDWR ) ) < 0 )
     {
        return( ER_LINK_CANT );
     }
   }

   return(ActiveLink);
}




/*
 *   Close Link
 */

int     CloseLink (link_t linkid)
{
   if ( linkid != ActiveLink )
      return( -1 );
   close(ActiveLink);
   ActiveLink = NULL_LINK;
   return(TRUE);
}





/*
 *   Read Link
 */

int     ReadLink (link_t linkid, char *buf, unsigned len, int timout)
{
  int nleft;                                 /* remaining bytes to be read */
  int nread;              /* read bytes during the last read() system call */
	
  for (nleft = len; nleft > 0;)
  {                     /* while the all requested datas has not been read */
    nread = read(linkid, buf, nleft);
    if (nread < 0)
    {
      fprintf(stderr, "readn: error reading from channel\n");
      exit(1);
    }
    nleft -= nread;
    buf += nread;
  }
  return (len - nleft);                                 
}




/*
 *   Write Link
 */

int     WriteLink (link_t linkid, char *buf, unsigned len, int timout)
{
  int nleft;                              /* remaining bytes to be written */
  int nwritten;       /* written bytes during the last write() system call */
  
  for (nleft = len; nleft > 0;)
  {                  /* while the all requested datas has not been written */
    nwritten = write(linkid, buf, nleft);
    if (nwritten < 0)
    {
      fprintf(stderr, "writen: error writing to channel\n");
      exit(1);
    }
    nleft -= nwritten;
    buf += nwritten;
  }
  return (len - nleft);
}



/*
 *   Reset Link
 */

int     ResetLink (link_t linkid)
{

   if ( linkid != ActiveLink )
      return( -1 );
   if ( ioctl( linkid, B004RESET, 1) < 0 )
      return( -1 );

   return( 1 );
}



/*
 *   Analyse Link
 */
 
int     AnalyseLink (link_t linkid)
{

   if ( linkid != ActiveLink )
      return( -1 );
   if ( ioctl( linkid, B004ANALYSE, 0) != 0 )
      return( -1 );

   return( 1 );
}



/*
 *   Test Error
 */
 
int     TestError (link_t linkid)
{
   if ( linkid != ActiveLink )
      return( -1 );

   return ( ioctl( linkid, B004ERROR, 0 ) );
}




/*
 *   Test Read
 */

int     TestRead (link_t linkid)
{
   if ( linkid != ActiveLink )
      return( -1 );

   return ( ioctl( linkid, B004READABLE, 0 ) );
}




/*
 *   Test Write
 */

int     TestWrite (link_t linkid)
{
   if ( linkid != ActiveLink )
      return( -1 );

   return ( ioctl( linkid, B004WRITEABLE, 0 ) );
}

/*
 *
 *      void WaitLink (bool_t wfl)
 *
 *  EXTENSION.  Indicates whether we want to wait for board to become free
 *  instead of fail if busy.
 *
 */

static  bool_t  waitforlink = FALSE;

void    WaitLink (wfl)
bool_t  wfl;
{
    waitforlink = wfl;
}


