/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* School of Engineering of Biel/Bienne, Switzerland.                      */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* Matrix utilities.                                                       */
/* Vector allocation routines. These routines can be used to allocate and  */
/* free vectors local to a processor. They are used when one of the tasks  */
/* of the network needs to allocate temporary data.                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 21.10.1994  Droux       Created                                         */
/* 12.01.1995  Droux       Defines NULL if unknown (useful for SUN4)       */
/*-------------------------------------------------------------------------*/
/* filename: vectoralloc.c                                                 */
/* Tested on :                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 School of Engineering of Biel/Bienne, Switzerland.   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <ppc/matrixutil/vectoralloc.h>

#ifndef NULL
#define NULL ((void *)0)
#endif

/*-------------------------------------------------------------------------*/
/* VECALLOC_BODY:                                                    MACRO */
/* Structure of a vector allocation routine.                               */
/*-------------------------------------------------------------------------*/
#define VECALLOC_BODY(type) { \
  type *vec; \
  \
  vec = (type *)malloc(n * sizeof(type)); \
  if (vec == NULL) \
    return NULL; \
  \
  return vec; \
}

/*-------------------------------------------------------------------------*/
/* dvecalloc:                                                              */
/* Allocates a n x m vector of doubles.                                    */
/*-------------------------------------------------------------------------*/
double *dvecalloc (
  int n
)
VECALLOC_BODY(double)

/*-------------------------------------------------------------------------*/
/* fvecalloc:                                                              */
/* Allocates a n x m vector of floats.                                     */
/*-------------------------------------------------------------------------*/
float *fvecalloc (
  int n
)
VECALLOC_BODY(float)

/*-------------------------------------------------------------------------*/
/* ivecalloc:                                                              */
/* Allocates a n x m vector of int.                                        */
/*-------------------------------------------------------------------------*/
int *ivecalloc (
  int n
)
VECALLOC_BODY(int)

/*-------------------------------------------------------------------------*/
/* dvecfree:                                                               */
/* Free the memory allocated for a double vector.                          */
/*-------------------------------------------------------------------------*/
void dvecfree (
  double *vec
)
{
  free(vec);
}

/*-------------------------------------------------------------------------*/
/* fvecfree:                                                               */
/* Free the memory allocated for a float vector.                           */
/*-------------------------------------------------------------------------*/
void fvecfree (
  float *vec
)
{
  free(vec);
}

/*-------------------------------------------------------------------------*/
/* ivecfree:                                                               */
/* Free the memory allocated for an integer vector.                        */
/*-------------------------------------------------------------------------*/
void ivecfree (
  int *vec
)
{
  free(vec);
}

