/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library Management.                                   */
/* Virtual machine configuration for a library client program.             */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 18.01.1995  Droux       Created                                         */
/* 29.03.1995  Droux       Added ring channels                             */
/* 03.04.1995  Droux       Free all channels                               */
/*-------------------------------------------------------------------------*/
/* filename: libvmc.c                                                      */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel                                                   */
/*   NEXTSTEP 3.3, Intel                                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define LIBVMC_IMPL

#include <stdio.h>
#include <ppc/portable.h>
#include <ppc/config.h>
#include <ppc/parlib/config.h>
#include <ppc/matrixutil.h>
#include "scallergen.h"
#include "mcallergen.h"

#define TEST_ALLOC(var) { if (var == NULL) \
  {printf("not enough memory to initialize virtual machine\n"); exit(1); }}

/*-------------------------------------------------------------------------*/
/* parlib_config:                                                          */
/* Configures a virtual machine for use of PPC parallel libraries.         */
/*-------------------------------------------------------------------------*/
#ifdef PACTC
void parlib_config (channel param_from_host, ...)
#else
void parlib_config (va_alist)
va_dcl
#endif
{
  va_list pvar;                  /* needed for variable arguements parsing */

  processor master_p;                        
  processor **slave_p;

  task master_t;                                                  
  task **slave_t;

  channel to_host;
  channel from_host;
  channel *master_to_slaves;
  channel *slaves_to_master;
  channel **left_to_right;
  channel **right_to_left;
  channel **up_to_down;
  channel **down_to_up;
  channel *lc_down;
  channel *lc_up;
  
  implementation master_i;
  implementation slave_i;
  char *act_module;
  int nb_modules;
  
  int nb_row;
  int nb_col;
  int nb_slaves;
  int i;
  int j;
  int k;
  int c;

  #ifdef PACTC
  va_start(pvar, param_from_host);
  from_host = param_from_host;
  #else
  va_start(pvar);
  from_host = va_arg(pvar, channel);
  #endif
  to_host = va_arg(pvar, channel);
  nb_row = va_arg(pvar, int);
  nb_col = va_arg(pvar, int);
  nb_slaves = nb_row * nb_col;

  slave_p = imatalloc(nb_row, nb_col); TEST_ALLOC(slave_p);
  slave_t = imatalloc(nb_row, nb_col); TEST_ALLOC(slave_t);
  master_to_slaves = ivecalloc(nb_slaves); TEST_ALLOC(master_to_slaves);
  slaves_to_master = ivecalloc(nb_slaves); TEST_ALLOC(slaves_to_master);
  left_to_right = imatalloc(nb_row+1, nb_col+1); TEST_ALLOC(left_to_right);
  right_to_left = imatalloc(nb_row+1, nb_col+1); TEST_ALLOC(right_to_left);
  up_to_down = imatalloc(nb_row+1, nb_col+1); TEST_ALLOC(up_to_down);
  down_to_up = imatalloc(nb_row+1, nb_col+1); TEST_ALLOC(down_to_up);
  lc_down = ivecalloc(nb_slaves); TEST_ALLOC(lc_down);
  lc_up = ivecalloc(nb_slaves); TEST_ALLOC(lc_up);

                                            /* initialize implementations */
  master_i = new_implementation(LIB_MCALLER_NAME, NULL);
  slave_i = new_implementation(LIB_SCALLER_NAME, NULL);

  act_module = va_arg(pvar, char *);
  for (nb_modules = 0; act_module != NULL; nb_modules++)
  {
    add_module(master_i, act_module);
    act_module = va_arg(pvar, char *);
  }
  if (nb_modules == 0)
  {
    fprintf(stderr, "configuration error: no user module defined\n");
    exit(1);
  }
  
  /* The files must be generated only if this configuration is called      */
  /* by the configurer, but not when running the program.                  */
  
  #ifdef PPC_CONFIGURER
  
  {                                              /* generates slave caller */
    char *act_lib;
    int nb_lib;
    
    scaller_gen_init();
    act_lib = va_arg(pvar, char *);
    for (nb_lib = 0; act_lib != NULL; nb_lib++)
    {
      scaller_gen_add_lib(act_lib);
      act_lib = va_arg(pvar, char *);
    }
    if (nb_lib == 0)
    {
      fprintf(stderr, "configuration error: no library defined\n");
      exit(1);
    }
    scaller_gen_generate();
  }
  
  mcaller_gen_generate();                       /* generates master caller */
  
  #endif
  
  /* initialize processors */
  master_p = new_processor();
  for (i = 0; i < nb_row; i++)
    for (j = 0; j < nb_col; j++)
      slave_p[i][j] = new_processor();
  
  /* initialize master-slave channels */
  for (c = 0; c < nb_slaves; c++)
  {
    master_to_slaves[c] = new_channel();
    slaves_to_master[c] = new_channel();
    lc_down[c] = new_channel();
    lc_up[c] = new_channel();
  }
  
  /* initialize inter-slave channels */
  for (i = 0; i < nb_row+1; i++)
  {
    for (j = 0; j < nb_col+1; j++)
    {
      left_to_right[i][j] = new_channel();
      right_to_left[i][j] = new_channel();
      up_to_down[i][j] = new_channel();
      down_to_up[i][j] = new_channel();
    }
  }

  /* initialize tasks */
  master_t = new_task(
    master_p,
    master_i,
    INPUT_CHANNEL, from_host,
    OUTPUT_CHANNEL, to_host,
    INPUT_CHANNEL_ARRAY, nb_slaves, slaves_to_master,
    OUTPUT_CHANNEL_ARRAY, nb_slaves, master_to_slaves,
    INT, nb_row,
    INT, nb_col,
    END);
  
  for (k = 0, i = 0; i < nb_row; i++)
  {
    for (j = 0; j < nb_col; j++, k++)
    {
      slave_t[i][j] = new_task (
        slave_p[i][j],
	slave_i,
	INPUT_CHANNEL, master_to_slaves[k],             /* from master */
	OUTPUT_CHANNEL, slaves_to_master[k],            /* to master */
	INPUT_CHANNEL, up_to_down[i][j],                /* from_north */
	OUTPUT_CHANNEL, down_to_up[i][j],               /* to north */
	INPUT_CHANNEL, down_to_up[i+1][j],              /* from south */
	OUTPUT_CHANNEL, up_to_down[i+1][j],             /* to south */
	INPUT_CHANNEL, left_to_right[i][j],             /* from west */
	OUTPUT_CHANNEL, right_to_left[i][j],            /* to west */
	INPUT_CHANNEL, right_to_left[i][j+1],           /* from east */
	OUTPUT_CHANNEL, left_to_right[i][j+1],          /* to east */
	INPUT_CHANNEL, lc_up[(k+1)%nb_slaves],          /* from next */
	OUTPUT_CHANNEL, lc_down[(k+1)%nb_slaves],       /* to next */
	INPUT_CHANNEL, lc_down[k],                      /* from prev */
	OUTPUT_CHANNEL, lc_up[k],                       /* to prev */
	INT, nb_row,
	INT, nb_col,
	INT, i,
	INT, j,
	END);
    }
  }

  hardware_config(master_p, slave_p);

  imatfree(slave_p, nb_row);
  imatfree(slave_t, nb_row);
  ivecfree(master_to_slaves);
  ivecfree(slaves_to_master);
  imatfree(left_to_right, nb_row+1);
  imatfree(right_to_left, nb_row+1);
  imatfree(up_to_down, nb_row+1);
  imatfree(down_to_up, nb_row+1);
  ivecfree(lc_down);
  ivecfree(lc_up);
}
