/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library Management.                                   */
/* Master routine caller generator.                                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 21.01.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: mcallergen.c                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel                                                   */
/*   NEXTSTEP 3.3, Intel                                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include <ppc/portable.h>
#include <ppc/csp/version.h>
#include "mcallergen.h"

/*-------------------------------------------------------------------------*/
/* mcaller_gen_generate:                                                   */
/* Generates the master caller.                                            */
/*-------------------------------------------------------------------------*/
void mcaller_gen_generate (void)
{
  FILE *out;
  
  gen_create_dir(".ppc", 0);
  
  out = fopen(LIB_MCALLER_NAME ".c", "w");
  if (out == NULL)
  {
    printf("could not create %s\n", LIB_MCALLER_NAME);
    exit(1);
  }
  
  fprintf(out, "\n");
  fprintf(out, "/* This file has been generated by the PPC ");
  fprintf(out, "configurer */\n");
  fprintf(out, "/* " PPC_VERSION " */\n");
  fprintf(out, "/* NEVER EDIT IT ! */\n");
  fprintf(out, "/* Created on %s by %s */\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "/* library client master caller */\n");
  fprintf(out, "\n");
  
  fprintf(out, "#define MKERNEL_CALLER\n");
  fprintf(out, "\n");
  
  fprintf(out, "#include <ppc/portable.h>\n");
  fprintf(out, "#include <ppc/parlib/mkernel.h>\n");
  fprintf(out, "#include <ppc/parlib/user.h>\n");
  fprintf(out, "\n");
  
  fprintf(out, "void ppc_main (\n");
  fprintf(out, "  channel from_host,\n");
  fprintf(out, "  channel to_host,\n");
  fprintf(out, "  channel from_slaves[],\n");
  fprintf(out, "  channel to_slaves[],\n");
  fprintf(out, "  int nb_row,\n");
  fprintf(out, "  int nb_col\n");
  fprintf(out, ")\n");
  fprintf(out, "{\n");
  fprintf(out, "  mkern_up(\n");
  fprintf(out, "    from_host,\n");
  fprintf(out, "    to_host,\n");
  fprintf(out, "    from_slaves,\n");
  fprintf(out, "    to_slaves,\n");
  fprintf(out, "    nb_row,\n");
  fprintf(out, "    nb_col);\n\n");
  fprintf(out, "  ppc_user_main();\n\n");
  fprintf(out, "  mkern_down();\n");
  fprintf(out, "}\n");

  fclose(out);
}


