/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library Management.                                   */
/* Slave routine caller generator.                                         */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 19.01.1995  Droux       Created                                         */
/* 29.03.1995  Droux       Added ring channels                             */
/*-------------------------------------------------------------------------*/
/* filename: scallergen.c                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel                                                   */
/*   NEXTSTEP 3.3, Intel                                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include <ppc/portable.h>
#include <ppc/csp/version.h>
#include "scallergen.h"

typedef struct _lib_name_struct
{
  char name [256];
  struct _lib_name_struct *next;
} lib_name_struct;

static lib_name_struct *lib_names = NULL;
static lib_name_struct *last_added;

/*-------------------------------------------------------------------------*/
/* scaller_gen_init:                                                       */
/* Initialize the caller generator module.                                 */
/*-------------------------------------------------------------------------*/
void scaller_gen_init (void)
{
  lib_names = NULL;
}

/*-------------------------------------------------------------------------*/
/* scaller_gen_add_lib:                                                    */
/* Add the given library to the list of libraries used by the program.     */
/*-------------------------------------------------------------------------*/
void scaller_gen_add_lib (
  char *lib_name
)
{
  lib_name_struct *ptr;
  
  if (lib_names == NULL)
    ptr = lib_names = malloc(sizeof(lib_name_struct));
  else
    ptr = lib_names->next = malloc(sizeof(lib_name_struct));
    
  strcpy(ptr->name, lib_name);
  ptr->next = NULL;

  last_added = ptr;
}

/*-------------------------------------------------------------------------*/
/* scaller_gen_generate:                                                   */
/* Generates the slave caller.                                             */
/*-------------------------------------------------------------------------*/
void scaller_gen_generate (void)
{
  FILE *out;
  
  gen_create_dir(".ppc", 0);

  out = fopen(LIB_SCALLER_NAME ".c", "w");
  if (out == NULL)
  {
    printf("could not create %s\n", LIB_SCALLER_NAME);
    exit(1);
  }
  
  fprintf(out, "\n");
  fprintf(out, "/* This file has been generated by the PPC ");
  fprintf(out, "configurer */\n");
  fprintf(out, "/* " PPC_VERSION " */\n");
  fprintf(out, "/* NEVER EDIT IT ! */\n");
  fprintf(out, "/* Created on %s by %s */\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "/* library client slave caller */\n");
  fprintf(out, "\n");
  
  fprintf(out, "#define SKERNEL_CALLER\n");
  fprintf(out, "\n");
  
  fprintf(out, "#include <ppc/portable.h>\n");
  fprintf(out, "#include <ppc/parlib/skernel.h>\n");
  fprintf(out, "\n");
  
  {
    lib_name_struct *act_lib;
    
    act_lib = lib_names;
    while (act_lib != NULL)
    {
      fprintf(out, "int %s_lib_starter(char *);\n", act_lib->name);
      act_lib = act_lib->next;
    }
  }
  fprintf(out, "\n");

  fprintf(out, "void ppc_main (\n");
  fprintf(out, "  channel from_master,\n");
  fprintf(out, "  channel to_master,\n");
  fprintf(out, "  channel from_north,\n");
  fprintf(out, "  channel to_north,\n");
  fprintf(out, "  channel from_south,\n");
  fprintf(out, "  channel to_south,\n");
  fprintf(out, "  channel from_west,\n");
  fprintf(out, "  channel to_west,\n");
  fprintf(out, "  channel from_east,\n");
  fprintf(out, "  channel to_east,\n");
  fprintf(out, "  channel from_next,\n");
  fprintf(out, "  channel to_next,\n");
  fprintf(out, "  channel from_prev,\n");
  fprintf(out, "  channel to_prev,\n");
  fprintf(out, "  int nb_row,\n");
  fprintf(out, "  int nb_col,\n");
  fprintf(out, "  int i,\n");
  fprintf(out, "  int j\n");
  fprintf(out, ")\n");
  fprintf(out, "{\n");

  fprintf(out, "  skern_up(\n");
  fprintf(out, "    from_master,\n");
  fprintf(out, "    to_master,\n");
  fprintf(out, "    from_north,\n");
  fprintf(out, "    to_north,\n");
  fprintf(out, "    from_south,\n");
  fprintf(out, "    to_south,\n");
  fprintf(out, "    from_west,\n");
  fprintf(out, "    to_west,\n");
  fprintf(out, "    from_east,\n");
  fprintf(out, "    to_east,\n");
  fprintf(out, "    from_next,\n");
  fprintf(out, "    to_next,\n");
  fprintf(out, "    from_prev,\n");
  fprintf(out, "    to_prev,\n");
  fprintf(out, "    nb_row,\n");
  fprintf(out, "    nb_col,\n");
  fprintf(out, "    i,\n");
  fprintf(out, "    j,\n");

  {
    lib_name_struct *act_lib;
    
    act_lib = lib_names;
    while (act_lib != NULL)
    {
      fprintf(out, "    %s_lib_starter,\n", act_lib->name);
      act_lib = act_lib->next;
    }
  }
  
  fprintf(out, "    NULL);\n\n");

  fprintf(out, "  skern_sleeper();\n\n");

  fprintf(out, "  skern_down();\n");
  fprintf(out, "}\n");
  fprintf(out, "\n");

  fclose(out);
}

/*-------------------------------------------------------------------------*/
/* scaller_gen_lib_list:                                                   */
/* Generates to stdandard output a list of the libraries to be linked      */
/* with the program. Called by the PPC programs makefile generators.       */
/*-------------------------------------------------------------------------*/
void scaller_gen_lib_list (
  FILE *out
)
{
  lib_name_struct *act_lib;
  
  fprintf(out, "  -L$(PPC_HOME)/parlib/$(ARCH)/$(HOST_ARCH) \\\n");

  act_lib = lib_names;
  while (act_lib != NULL)
  {
    fprintf(out, "  -l%s \\\n", 
      act_lib->name);
    act_lib = act_lib->next;
  }  
}

