/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* DDS directory.                                                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 04.04.1995  Droux       Created                                         */
/* 06.05.1995  Droux       Added complex and boolean types                 */
/*-------------------------------------------------------------------------*/
/* filename: ddsdir.c                                                      */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define DDS_IMPL
#define DDS_MGMT_IMPL
#define MKERNEL_IMPL
#define DDS_DIR_IMPL

#include <stdlib.h>
#include <ppc/portable.h>
#include <ppc/parlib/mkernel.h>
#include <ppc/parlib/dds.h>
#include <ppc/parlib/ddspriv.h>
#include "ddsutil.h"

char *types_names_str[] =
{
  "INT",
  "SHORT",
  "UNSIGNED_SHORT",
  "LONG",
  "UNSIGNED_LONG",
  "FLOAT",
  "DOUBLE",
  "CHAR",
  "COMPLEX",
  "BOOLEAN"
};

char *dds_distribution_str[] =
{
  "DDS_REPLICATED"
};

static dds_object *act_object_prop;

/*-------------------------------------------------------------------------*/
/* dds_directory_init:                                                     */
/* Must be called before reading the DDS objects directory.                */
/*-------------------------------------------------------------------------*/
void dds_directory_init (void)
{
  act_object_prop = ddsi_master_get_first_object();
}

/*-------------------------------------------------------------------------*/
/* dds_directory_get_entry:                                                */
/* Get the next DDS object entry. Returns non-zero if no more objects are  */
/* available, 0 otherwise.                                                 */
/*-------------------------------------------------------------------------*/
int dds_directory_get_entry (
  dds_object_entry *entry
)
{
  int c;
  
  if (act_object_prop == NULL)
    return 1;
    
  strcpy(entry->name, act_object_prop->name);
  entry->type = act_object_prop->type;
  entry->distribution = act_object_prop->distribution;
  entry->nb_dim = act_object_prop->nb_dim;
  for (c = 0; c < act_object_prop->nb_dim; c++)
    entry->dim[c] = act_object_prop->dim[c];
    
  act_object_prop = act_object_prop->next;
    
  return 0;
}

/*-------------------------------------------------------------------------*/
/* dds_object_prop:                                                        */
/* Return the properties of the specified object. Returns -1 if the        */
/* object cannot be found, 0 otherwise.                                    */
/*-------------------------------------------------------------------------*/
int dds_object_prop (
  char *name,
  dds_object_entry *entry
)
{
  dds_object *object;
  int c;

  if ((object = ddsi_find_object(ddsi_master_get_first_object(), name)) 
      == NULL)
    return -1;                                         /* object not found */

  strcpy(entry->name, object->name);
  entry->type = object->type;
  entry->distribution = object->distribution;
  entry->nb_dim = object->nb_dim;
  for (c = 0; c < object->nb_dim; c++)
    entry->dim[c] = object->dim[c];
    
  return 0;
}

