/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* DDS master routines.                                                    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.03.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: ddsmaster.c                                                   */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define DDS_IMPL
#define DDS_MASTER_IMPL

#include <stdlib.h>
#include <ppc/portable.h>
#include <ppc/parlib/dds.h>
#include <ppc/parlib/ddspriv.h>
#include <ppc/parlib/mkernel.h>
#include <ppc/csp/chan.h>
#include "ddsutil.h"

static dds_object *dds_master_objects_first;
static dds_object *dds_master_objects_last;

/*-------------------------------------------------------------------------*/
/* ddsi_master_send_object_prop:                                           */
/* Send the given object property to the slaves.                           */
/*-------------------------------------------------------------------------*/
void ddsi_master_send_object_prop (
  dds_object *object
)
{
  int c;
  
  for (c = 0; c < nb_slaves; c++)
    send_channel(to_slaves[c],
      INT, object->type,
      INT, object->distribution,
      INT, object->nb_dim,
      STRING, DDS_MAX_NAME_LEN, object->name,
      INT_ARRAY, DDS_MAX_DIM, object->dim,
      END);
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_init:                                                       */
/* Initialize the DDS. Called by the master.                               */
/*-------------------------------------------------------------------------*/
void ddsi_master_init (void)
{
  dds_master_objects_first = NULL;
  dds_master_objects_last = NULL;
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_terminate:                                                  */
/* Terminate the DDS. Called by the master.                                */
/*-------------------------------------------------------------------------*/
void ddsi_master_terminate (void)
{
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_add_object:                                                 */
/* Add given object to master's objects list.                              */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_add_object (
  dds_object *object
)
{
  ddsi_add_object(
    &dds_master_objects_first,
    &dds_master_objects_last,
    object);
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_clear_database:                                             */
/* Clear master's objects database.                                        */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_clear_database (void)
{
  ddsi_clear_database(
    &dds_master_objects_first, 
    &dds_master_objects_last);
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_remove_object:                                              */
/* Remove object from master's object list.                                */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
void ddsi_master_remove_object (
  dds_object *object
)
{
  ddsi_remove_object(
    &dds_master_objects_first,
    &dds_master_objects_last,
    object);
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_get_first_object:                                           */
/* Return the first object in the objects list.                            */
/* ddsmaster.c                                                             */
/*-------------------------------------------------------------------------*/
dds_object *ddsi_master_get_first_object (void)
{
  return dds_master_objects_first;
}

/*-------------------------------------------------------------------------*/
/* ddsi_master_send_req:                                                   */
/* Send a DDS request to the slaves.                                       */
/*-------------------------------------------------------------------------*/
void ddsi_master_send_req (
  DDSI_REQ req
)
{
  int c;
  
  mkern_dds_req();
  
  for (c = 0; c < nb_slaves; c++)
    send_channel(to_slaves[c],
      INT, (int)req,
      END);
}

/*-------------------------------------------------------------------------*/
/* dds_send_name:                                                          */
/* Send object name to slaves. Each slave must call the function           */
/* dds_receive_name for each call to this function.                        */
/* For master only.                                                        */
/*-------------------------------------------------------------------------*/
void dds_send_name (
  char *name
)
{
  int c;
  
  for (c = 0; c < nb_slaves; c++)
    send_channel(to_slaves[c],
      STRING, DDS_MAX_NAME_LEN, name,
      END);
}

/*-------------------------------------------------------------------------*/
/* dds_master_object_ptr:                                                  */
/* Return a pointer to the specified object.                               */
/*-------------------------------------------------------------------------*/
void *dds_master_object_ptr (
  char *name
)
{
  dds_object *object;
  
  object = ddsi_find_object(ddsi_master_get_first_object(), name);
  if (object == NULL)
    return object;
  
  return object->value;
}

