/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* DDS functions for routines.                                             */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 05.05.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: ddsroutine.c                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define DDS_IMPL

#include <stdlib.h>
#include <ppc/portable.h>
#include <ppc/parlib/dds.h>
#include <ppc/parlib/ddspriv.h>
#include <ppc/parlib/mkernel.h>
#include <ppc/parlib/skernel.h>
#include <ppc/csp/chan.h>
#include "ddsutil.h"

/*-------------------------------------------------------------------------*/
/* dds_object_ptr:                                                         */
/* Return a pointer to the specified object.                               */
/* For master and slaves.                                                  */
/*-------------------------------------------------------------------------*/
void *dds_object_ptr (
  char *name
)
{
  dds_object *object;
  
  object = ddsi_find_object(ddsi_slave_get_first_object(), name);
  if (object == NULL)
    return object;
  
  return object->value;
}

/*-------------------------------------------------------------------------*/
/* dds_receive_name:                                                       */
/* Get an object name from the master. Each slave must call this function  */
/* for each dds_send_name master call.                                     */
/* The returned pointer is valid until the next call to this function.     */
/* For slaves only.                                                        */
/*-------------------------------------------------------------------------*/
char *dds_receive_name (void)
{
  static char name[DDS_MAX_NAME_LEN];
  
  receive_channel(from_master,
    STRING, DDS_MAX_NAME_LEN, name,
    END);
  
  return name;
}

/*-------------------------------------------------------------------------*/
/* dds_send_name:                                                          */
/* Send object name to slaves. Each slave must call the function           */
/* dds_receive_name for each call to this function.                        */
/* For master only.                                                        */
/*-------------------------------------------------------------------------*/
void dds_send_name (
  char *name
)
{
  int c;
  
  for (c = 0; c < nb_slaves; c++)
    send_channel(to_slaves[c],
      STRING, DDS_MAX_NAME_LEN, name,
      END);
}

