/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* DDS utilities used by both the master and the slaves.                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 05.04.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: ddsutil.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef DDS_UTIL_H
#define DDS_UTIL_H

#include <stdio.h>

/* In ddsi_value_copy(), we can copy only one part of an object into       */
/* another one (DDSI_COPY_IDX_SRC), or copy an object into a part of       */
/* another one (DDSI_COPY_IDX_DEST).                                       */
typedef enum
{
  DDSI_COPY_IDX_SRC,
  DDSI_COPY_IDX_DEST,
  DDSI_COPY_IDX_NONE
} DDSI_COPY_IDX;

/*-------------------------------------------------------------------------*/
/* ddsi_add_object:                                                        */
/* Add given object to objects list.                                       */
/*-------------------------------------------------------------------------*/
void ddsi_add_object (
  dds_object **first,
  dds_object **last,
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_remove_object:                                                     */
/* Remove object from object list.                                         */
/*-------------------------------------------------------------------------*/
void ddsi_remove_object (
  dds_object **first,
  dds_object **last,
  dds_object *object
);

/*-------------------------------------------------------------------------*/
/* ddsi_find_object:                                                       */
/* Return a pointer to the given object, or NULL if object not found.      */
/*-------------------------------------------------------------------------*/
dds_object *ddsi_find_object (
  dds_object *first,
  char *name
);

/*-------------------------------------------------------------------------*/
/* ddsi_free_value:                                                        */
/* Release the memory used by an object value.                             */
/*-------------------------------------------------------------------------*/
void ddsi_free_value (
  void *value,
  int act_dim,
  int nb_dim,
  int *dim
);

/*-------------------------------------------------------------------------*/
/* ddsi_clear_database:                                                    */
/* Clear objects.                                                          */
/*-------------------------------------------------------------------------*/
void ddsi_clear_database (
  dds_object **first,
  dds_object **last
);

/*-------------------------------------------------------------------------*/
/* ddsi_value_allocate:                                                    */
/* Allocate memory for an object value.                                    */
/*-------------------------------------------------------------------------*/
void *ddsi_value_allocate (
  types_names type,
  int act_dim,
  int nb_dim,
  int *dim
);

/*-------------------------------------------------------------------------*/
/* ddsi_value_copy:                                                        */
/* Copy the value of an object into another one                            */
/*-------------------------------------------------------------------------*/
void ddsi_value_copy (
  void *src_value,
  void *dest_value,
  int type,
  int act_dim,
  int nb_dim,
  int *dim,
  int nb_idx,
  int *idx,
  DDSI_COPY_IDX to_apply
);

/*-------------------------------------------------------------------------*/
/* ddsi_print_vector:                                                      */
/* Print the given vector on the specified stream.                         */
/*-------------------------------------------------------------------------*/
void ddsi_print_vector (
  FILE *stream,
  int type,
  int dim,
  void *value
);

/*-------------------------------------------------------------------------*/
/* ddsi_scan_vector:                                                       */
/* Scan the given vector from the specified stream.                        */
/*-------------------------------------------------------------------------*/
void ddsi_scan_vector (
  FILE *stream,
  int type,
  int dim,
  void *value
);

#endif
