/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel Library manager master kernel.                                 */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 21.01.1995  Droux       Created                                         */
/* 28.03.1995  Droux       Added DDS initialization                        */
/* 29.03.1995  Droux       Added mkern_dds_req()                           */
/*-------------------------------------------------------------------------*/
/* filename: mkernel.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define MKERNEL_IMPL

#include <stdlib.h>
#include <ppc/portable.h>
#include <ppc/csp/types.h>
#include <ppc/parlib/mkernel.h>
#include <ppc/parlib/ddspriv.h>
#include "kernproto.h"

channel mkern_to_host;
channel mkern_from_host;
channel *mkern_to_slaves;
channel *mkern_from_slaves;
int mkern_nb_slaves;
int mkern_nb_row;
int mkern_nb_col;

/*-------------------------------------------------------------------------*/
/* mkern_up:                                                               */
/* Initializes the library master kernel.                                  */
/* Called by the master starter generated by the PPC configurer.           */
/*-------------------------------------------------------------------------*/
void mkern_up (
  channel from_host,
  channel to_host,
  channel from_slaves[],
  channel to_slaves[],
  int nb_row,
  int nb_col
)
{
  int c;
  
  mkern_to_host = to_host;
  mkern_from_host = from_host;
  mkern_nb_row = nb_row;
  mkern_nb_col = nb_col;
  mkern_nb_slaves = nb_row*nb_col;
  
  mkern_to_slaves = (channel *)malloc(mkern_nb_slaves * sizeof(channel));
  mkern_from_slaves = (channel *)malloc(mkern_nb_slaves * sizeof(channel));
  if (mkern_to_slaves == NULL || mkern_from_slaves == NULL)
  {
    printf("PPC master kernel: error allocating channels\n");
    exit(1);
  }

  for (c = 0; c < mkern_nb_slaves; c++)
  {
    mkern_to_slaves[c] = to_slaves[c];
    mkern_from_slaves[c] = from_slaves[c];
  }
  
  ddsi_master_init();
}

/*-------------------------------------------------------------------------*/
/* mkern_start_routine:                                                    */
/* Starts the specified routine on the slaves.                             */
/*-------------------------------------------------------------------------*/
void mkern_start_routine (
  char *routine_name
)
{
  char rname [PPR_ROUTINE_NAME_LEN];
  int c;
  
  strcpy(rname, routine_name);
  
  for (c = 0; c < mkern_nb_slaves; c++)
  {
    send_channel(mkern_to_slaves[c],
      INT, PARLIB_KERN_CALL,
      END);
  
    send_channel(mkern_to_slaves[c],
      STRING, PPR_ROUTINE_NAME_LEN, routine_name,
      END);
  }
}

/*-------------------------------------------------------------------------*/
/* mkern_down:                                                             */
/* Shutdown the master kernel.                                             */
/*-------------------------------------------------------------------------*/
void mkern_down (void)
{
  int c;
  
  for (c = 0; c < mkern_nb_slaves; c++)
  {                                /* send terminate request to all slaves */
    send_channel(mkern_to_slaves[c],
      INT, PARLIB_KERN_TERMINATE,
      END);
  }

  ddsi_master_terminate();

  free(mkern_to_slaves);
  free(mkern_from_slaves);
}

/*-------------------------------------------------------------------------*/
/* mkern_dds_req:                                                          */
/* Send DDS request to slave kernels.                                      */
/*-------------------------------------------------------------------------*/
void mkern_dds_req (void)
{
  int c;
  
  for (c = 0; c < mkern_nb_slaves; c++)
  {                                
    send_channel(mkern_to_slaves[c],
      INT, PARLIB_KERN_DDS,
      END);
  }
}

/*-------------------------------------------------------------------------*/
/* mkern_dds_req_to:                                                       */
/* Send DDS request to a specific slave kernel.                            */
/*-------------------------------------------------------------------------*/
void mkern_dds_req_to (
  int slave_id
)
{
  send_channel(mkern_to_slaves[slave_id],
    INT, PARLIB_KERN_DDS,
    END);
}
