/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel Library slave kernel.                                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 21.01.1995  Droux       Created                                         */
/* 28.03.1995  Droux       Added DDS initialization                        */
/* 29.03.1995  Droux       Added support for PARLIB_KERN_DDS               */
/* 29.03.1995  Droux       Added ring channels initialization              */
/* 03.04.1995  Droux       Local functions are declared as static          */
/*-------------------------------------------------------------------------*/
/* filename: skernel.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define SKERNEL_IMPL

#include <stdlib.h>
#include <ppc/portable.h>
#include <ppc/csp/types.h>
#include <ppc/parlib/skernel.h>
#include <ppc/parlib/ddspriv.h>
#include "kernproto.h"

typedef struct _slave_starters_struct
{
  int (* slave_starter)(char *);
  struct _slave_starters_struct *next;
} slave_starters_struct;

channel skern_to_master;                                
channel skern_from_master;
channel skern_to_next;                                  
channel skern_from_next;
channel skern_to_prev;
channel skern_from_prev;
channel skern_to_north;                                  
channel skern_from_north;
channel skern_to_south;
channel skern_from_south;
channel skern_to_west;
channel skern_from_west;
channel skern_to_east;
channel skern_from_east;
int skern_nb_slaves;                          
int skern_my_nb;
int skern_nb_row;                                        
int skern_nb_col;
int skern_my_row;
int skern_my_col;

slave_starters_struct *slave_starters = NULL;

/*-------------------------------------------------------------------------*/
/* skern_add_slave_starter:                                                */
/* Add the given function to the list of slave starters.                   */
/*-------------------------------------------------------------------------*/
void skern_add_slave_starter (
  int (* starter)(char *)
)
{
  static slave_starters_struct *last_added = NULL;
  slave_starters_struct *ptr;
  
  if (last_added == NULL)
    ptr = slave_starters = 
      (slave_starters_struct *) malloc(sizeof(slave_starters_struct));
  else
    ptr = last_added->next = 
      (slave_starters_struct *) malloc(sizeof(slave_starters_struct));
    
  ptr->slave_starter = starter;
  ptr->next = NULL;
  
  last_added = ptr;
}

/*-------------------------------------------------------------------------*/
/* skern_up:                                                               */
/* Initializes the library slave kernel.                                   */
/* Called by the slave starter generated by the PPC configurer.            */
/*-------------------------------------------------------------------------*/
#ifdef PACTC
void skern_up (channel from_master_param, ...)
#else
void skern_up (va_alist)
va_dcl
#endif
{
  va_list pvar;                  /* needed for variable arguements parsing */
  int (* act_starter)(char *);

  #ifdef PACTC
  va_start(pvar, from_master_param);
  skern_from_master = from_master_param;
  #else
  va_start(pvar);
  skern_from_master = va_arg(pvar, channel);
  #endif
  
  skern_to_master = va_arg(pvar, channel);
  skern_from_north = va_arg(pvar, channel);
  skern_to_north = va_arg(pvar, channel);
  skern_from_south = va_arg(pvar, channel);
  skern_to_south = va_arg(pvar, channel);
  skern_from_west = va_arg(pvar, channel);
  skern_to_west = va_arg(pvar, channel);
  skern_from_east = va_arg(pvar, channel);
  skern_to_east = va_arg(pvar, channel);
  skern_from_next = va_arg(pvar, channel);
  skern_to_next = va_arg(pvar, channel);
  skern_from_prev = va_arg(pvar, channel);
  skern_to_prev = va_arg(pvar, channel);
  skern_nb_row = va_arg(pvar, int);
  skern_nb_col = va_arg(pvar, int);
  skern_my_row = va_arg(pvar, int);
  skern_my_col = va_arg(pvar, int);
  
  skern_nb_slaves = skern_nb_row * skern_nb_col;

  #if 0  
  skern_my_nb = skern_my_row*skern_nb_col + 
    (skern_my_row%2)*(skern_nb_col-skern_my_col-1) +
    ((skern_my_row+1)%2)*skern_my_col;
  #else
  skern_my_nb = skern_my_row*skern_nb_col + skern_my_col;
  #endif
  
  act_starter = va_arg(pvar, void *);
  while (act_starter != NULL)
  {
    skern_add_slave_starter(act_starter);
    act_starter = va_arg(pvar, void *);
  }
  
  ddsi_slave_init();
}

/*-------------------------------------------------------------------------*/
/* skern_call:                                                       LOCAL */
/* Called when a call request has been received from master.               */
/*-------------------------------------------------------------------------*/
static void skern_call (void)
{
  char rname [PPR_ROUTINE_NAME_LEN];
  slave_starters_struct *act_starter;
  int found;
  
  receive_channel(skern_from_master,
      STRING, PPR_ROUTINE_NAME_LEN, rname,
      END);
      
  act_starter = slave_starters;
  found = 0;
  while ((act_starter != NULL) && !found)
  {
    found = !((act_starter->slave_starter)(rname));
    act_starter = act_starter->next;
  }

  send_channel(skern_to_master,            /* send terminate ack to master */
    INT, skern_my_nb,
    END);
}

/*-------------------------------------------------------------------------*/
/* skern_sleeper:                                                          */
/* Responds to master requests.                                            */
/*-------------------------------------------------------------------------*/
void skern_sleeper (void)
{
  int terminate = 0;
  int request;
  
  do
  {
    receive_channel(skern_from_master,
      INT, &request,
      END);
    
    switch(request)
    {
      case PARLIB_KERN_CALL:
	skern_call();
        break;
      case PARLIB_KERN_TERMINATE:
        terminate = 1;
        break;
      case PARLIB_KERN_DDS:
        ddsi_slave_handler();
        break;
      default:
        printf("slavekernel%d: got unknown request (%d), ignored\n",
	  skern_my_nb, request);
        break;
    }
  } while (!terminate);
}

/*-------------------------------------------------------------------------*/
/* skern_down:                                                             */
/* Shutdown the slave kernel.                                              */
/*-------------------------------------------------------------------------*/
void skern_down (void)
{
  ddsi_slave_terminate();
}
