/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library command line utility.                         */
/* Files generator.                                                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 13.12.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: lgenerator.c                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel                                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>

#include <ppc/portable.h>
#include <ppc/csp/version.h>
#include "lgenerator.h"
#include "lprop.h"
#include "genutil.h"
#include "lgendefs.h"
#include "rgendefs.h"

/*-------------------------------------------------------------------------*/
/* lgen_makefile:                                                          */
/* Generates the library makefile.                                         */
/*-------------------------------------------------------------------------*/
int lgen_makefile (
  int verbose
)
{
  char filename [256];
  FILE *out;
  
  if (verbose)
    printf("generating %s library makefile\n", lprop_lib_name());
    
  gen_create_dir(PPL_DIR, verbose);
  
  sprintf(filename, "%s/makefile", PPL_DIR);
  out = fopen(filename, "w");
  if (out == NULL)
    return -1;
    
  fprintf(out, "\n");
  fprintf(out, "# This file has been generated by the PPC ");
  fprintf(out, "library manager\n");
  fprintf(out, "# " PPC_VERSION "\n");
  fprintf(out, "# NEVER EDIT IT !\n");
  fprintf(out, "# Created on %s by %s\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "# %s library makefile\n",
    lprop_lib_name());
  fprintf(out, "\n");
  
  {                                               /* list routines headers */
    char *act_routine;
    
    fprintf(out, "USERHEADERS=");
    
    lprop_get_routine_init();
    act_routine = lprop_get_routine_next();
    while(act_routine != NULL)
    {
      fprintf(out, "%s/" PPR_DIR "/user.h ", act_routine);
      act_routine = lprop_get_routine_next();
    }
    fprintf(out, "\n");
  }
  
  fprintf(out, "USERHEADER=" PPL_DIR "/%s.h\n", lprop_lib_name());
  fprintf(out, "\n");
  
  fprintf(out, "headers: $(USERHEADER)\n");
  fprintf(out, "\n");
  
  fprintf(out, "$(USERHEADER): $(USERHEADERS)\n");
  fprintf(out, "\tcat $(USERHEADERS) > $@\n\n");
  
  fclose(out);
   
  if (verbose)
    printf("%s library makefile generated\n", lprop_lib_name());
    
  return 0;
}

/*-------------------------------------------------------------------------*/
/* lgen_lib_starter:                                                       */
/* Generates the library starter.                                          */
/*-------------------------------------------------------------------------*/
int lgen_lib_starter (
  int verbose
)
{
  char filename [256];
  FILE *out;
  
  if (verbose)
    printf("generating %s library starter\n", lprop_lib_name());
    
  gen_create_dir(PPL_DIR, verbose);
  
  sprintf(filename, "%s/_%sstarter.c", PPL_DIR, lprop_lib_name());
  out = fopen(filename, "w");
  if (out == NULL)
    return -1;
    
  fprintf(out, "\n");
  fprintf(out, "/* This file has been generated by the PPC ");
  fprintf(out, "routines manager */\n");
  fprintf(out, "/* " PPC_VERSION " */\n");
  fprintf(out, "/* NEVER EDIT IT ! */\n");
  fprintf(out, "/* Created on %s by %s */\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "/* %s library starter */\n",
    lprop_lib_name());
  fprintf(out, "\n");
  
  {                                   /* list routines starters prototypes */
    char *act_routine;
    
    fprintf(out, "/* routines starters prototypes */\n");
    lprop_get_routine_init();
    act_routine = lprop_get_routine_next();
    while(act_routine != NULL)
    {
      fprintf(out, "int %s_slave_starter(char *name);\n", act_routine);
      act_routine = lprop_get_routine_next();
    }
    fprintf(out, "\n");
  }

  fprintf(out, "/* library starter */\n");
  fprintf(out, "int %s_lib_starter (\n", lprop_lib_name());
  fprintf(out, "  char *name\n");
  fprintf(out, ")\n");
  fprintf(out, "{\n");
  
  {                                              /* list routines starters */
    char *act_routine;
    
    lprop_get_routine_init();
    act_routine = lprop_get_routine_next();
    while(act_routine != NULL)
    {
      fprintf(out, "  if (!%s_slave_starter(name))\n", act_routine);
      fprintf(out, "    return 0;\n");
      act_routine = lprop_get_routine_next();
    }
    fprintf(out, "\n");
  }
  
  fprintf(out, "  return -1;\n");
  fprintf(out, "}\n");

  fclose(out);
    
  if (verbose)
    printf("%s library starter generated\n", lprop_lib_name());

  return 0;
}

/*-------------------------------------------------------------------------*/
/* lgen_object_list:                                                       */
/* Generates in the specified stream a list of the object files for the    */
/* given routine.                                                          */
/*-------------------------------------------------------------------------*/
int lgen_object_list (
  FILE *out,
  char *routine,
  char *arch,
  char *host_arch
)
{
  FILE *in;
  char filename [256];
  char act_file [256];
  
  sprintf(filename, "%s/" PPR_DIR "/" GEN_OBJECTS_FILE, routine);
  in = fopen(filename, "r");
  if (in == NULL)
    return -1;
 
  fgets(act_file, 256, in);
  while (strcmp(act_file, "\n"))
  {
    act_file[strlen(act_file)-1] = '\0';
    fprintf(out, " \\\n");
    fprintf(out, "  %s/" PPR_DIR "/.%s/%s/%s.o",
      routine, arch, host_arch, act_file);
    fgets(act_file, 256, in);
  }
  
  fclose(in);
  
  return 0;
}

