/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* Library generator. This module can generate the files needed to         */
/* build a parallel library for the Paragon.                               */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 20.03.1995  Droux       Created                                         */
/* 01.05.1995  Droux       Local functions are declared as static          */
/*-------------------------------------------------------------------------*/
/* filename: lgenunix.c                                                    */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>

#include <ppc/portable.h>
#include <ppc/csp/version.h>
#include "lgenpgon.h"
#include "genutil.h"
#include "lprop.h"
#include "toolsutil.h"
#include "lgendefs.h"
#include "lgenerator.h"

/*-------------------------------------------------------------------------*/
/* lgen_pgon_makefile:                                               LOCAL */
/* Generates Paragon makefile.                                             */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
static int lgen_pgon_makefile (
  int verbose
)
{
  FILE *out;
  char filename[256];
  
  if (verbose)
    printf("generating Paragon library makefile\n");
  
  if (gen_create_dir(PPL_DIR, verbose) == -1)
    return -1;
    
  sprintf(filename, "%s/.%s/makefile", PPL_DIR, arch_names[PARAGON_ARCH]);
  out = fopen(filename, "w");
  if (out == NULL)
    return -1;
      
  fprintf(out, "\n");
  fprintf(out, "# This file has been generated by the PPC ");
  fprintf(out, "library manager\n");
  fprintf(out, "# " PPC_VERSION "\n");
  fprintf(out, "# NEVER EDIT IT !\n");
  fprintf(out, "# Created on %s by %s\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "# %s Paragon library makefile\n",
    lprop_lib_name());
  fprintf(out, "\n");

  fprintf(out, "CFLAGS = -I$(PPC_HOME)/include -D_$(ARCH) ");
  fprintf(out, "-D$(HOST_ARCH) -O $(OTHER_CFLAGS) \\\n");
  fprintf(out, "  $(WARNING_FLAG)\n\n");
  
  fprintf(out, "LIBOBJ = %s/.%s/$(HOST_ARCH)/_%sstarter.o", 
    PPL_DIR, arch_names[PARAGON_ARCH], lprop_lib_name());
  
  {                                      /* list of routines object files */
    int c;
    char *act_routine;
    
    lprop_get_routine_init();
    act_routine = lprop_get_routine_next();
    for (c = 0; act_routine != NULL; c++)
    {
      lgen_object_list(out, act_routine, 
        arch_names[PARAGON_ARCH], "$(HOST_ARCH)");
      act_routine = lprop_get_routine_next();
    }
  } 
  fprintf(out, "\n\n");
  
  fprintf(out, PPL_DIR "/.%s/$(HOST_ARCH)/lib%s.a: $(LIBOBJ)\n",
    arch_names[PARAGON_ARCH], lprop_lib_name());
  fprintf(out, "\tar r $@ $(LIBOBJ)\n");
  fprintf(out, "\tppc_ranlib $@\n\n");
  
  fprintf(out, "%s/.%s/$(HOST_ARCH)/_%sstarter.o: %s/_%sstarter.c\n", 
    PPL_DIR, arch_names[PARAGON_ARCH], lprop_lib_name(), 
    PPL_DIR, lprop_lib_name());
  fprintf(out, "\t$(CC) $(CFLAGS) -o $@ -c $?\n\n");

  fclose(out);
  
  if (verbose)
    printf("Paragon library makefile generated\n");
  
  return 0;
}

/*-------------------------------------------------------------------------*/
/* lgen_pgon_files:                                                        */
/* Generates files for Paragon.                                            */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int lgen_pgon_files (
  int verbose
)
{
  char dir[256];
  
  if (verbose)
    printf("generating Paragon library files\n");
  
  if (gen_create_dir(PPL_DIR, verbose) == -1)
    return -1;
    
  sprintf(dir, "%s/.%s", PPL_DIR, arch_names[PARAGON_ARCH]);
  if (gen_create_dir(dir, verbose) == -1)
    return -1;
  
  if (lgen_pgon_makefile(verbose) < 0)
    return -1;
    
  if (verbose)
    printf("Paragon library files generated\n");
  
  return 0;
}

