/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* Library generator. This module can generate the files needed to         */
/* build a parallel library for PVM.                                       */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.01.1995  Droux       Created                                         */
/* 18.04.1995  Droux       Changed UNIX_ARCH to PVM_ARCH                   */
/* 01.05.1995  Droux       Local functions are declared as static          */
/* 16.05.1995  Droux       Ported to the SP-2                              */
/*-------------------------------------------------------------------------*/
/* filename: lgenpvm.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>

#include <ppc/portable.h>
#include <ppc/csp/version.h>
#include "lgenpvm.h"
#include "genutil.h"
#include "lprop.h"
#include "toolsutil.h"
#include "lgendefs.h"
#include "lgenerator.h"

#ifdef SP2
  #define ARCH_DIR arch_names[SP2_ARCH]
#else
  #define ARCH_DIR arch_names[PVM_ARCH]
#endif

/*-------------------------------------------------------------------------*/
/* lgen_pvm_makefile:                                                LOCAL */
/* Generates PVM makefile.                                                 */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
static int lgen_pvm_makefile (
  int verbose
)
{
  /* For the moment, the slave and master code are stored in the same      */
  /* library. This is not optimal but much more simpler.                   */

  FILE *out;
  char filename[256];
  
  if (verbose)
    printf("generating PVM library makefile\n");
  
  if (gen_create_dir(PPL_DIR, verbose) == -1)
    return -1;
    
  sprintf(filename, "%s/.%s/makefile", PPL_DIR, ARCH_DIR);
  out = fopen(filename, "w");
  if (out == NULL)
    return -1;
      
  fprintf(out, "\n");
  fprintf(out, "# This file has been generated by the PPC ");
  fprintf(out, "library manager\n");
  fprintf(out, "# " PPC_VERSION "\n");
  fprintf(out, "# NEVER EDIT IT !\n");
  fprintf(out, "# Created on %s by %s\n", 
    gen_get_date(), gen_get_user_name());
  fprintf(out, "# %s PVM library makefile\n",
    lprop_lib_name());
  fprintf(out, "\n");

  fprintf(out, "CFLAGS = -I$(PPC_HOME)/include -D_$(ARCH) ");
  fprintf(out, "-D$(HOST_ARCH) -O $(OTHER_CFLAGS) \\\n");
  fprintf(out, "  $(WARNING_FLAG)\n\n");
  
  fprintf(out, "LIBOBJ = %s/.%s/$(HOST_ARCH)/_%sstarter.o", 
    PPL_DIR, ARCH_DIR, lprop_lib_name());
  
  {                                      /* list of routines object files */
    int c;
    char *act_routine;
    
    lprop_get_routine_init();
    act_routine = lprop_get_routine_next();
    for (c = 0; act_routine != NULL; c++)
    {
      lgen_object_list(out, act_routine, 
        ARCH_DIR, "$(HOST_ARCH)");
      act_routine = lprop_get_routine_next();
    }
  } 
  fprintf(out, "\n\n");
  
  fprintf(out, PPL_DIR "/.%s/$(HOST_ARCH)/lib%s.a: $(LIBOBJ)\n",
    ARCH_DIR, lprop_lib_name());
  fprintf(out, "\tar r $@ $(LIBOBJ)\n");
  fprintf(out, "\tppc_ranlib $@\n\n");
  
  fprintf(out, "%s/.%s/$(HOST_ARCH)/_%sstarter.o: %s/_%sstarter.c\n", 
    PPL_DIR, ARCH_DIR, lprop_lib_name(), 
    PPL_DIR, lprop_lib_name());
  fprintf(out, "\t$(CC) $(CFLAGS) -o $@ -c $?\n\n");

  fclose(out);
  
  if (verbose)
    printf("PVM library makefile generated\n");
  
  return 0;
}

/*-------------------------------------------------------------------------*/
/* lgen_pvm_files:                                                         */
/* Generates files for PVM.                                                */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int lgen_pvm_files (
  int verbose
)
{
  char dir[256];
  
  if (verbose)
    printf("generating PVM library files\n");
  
  if (gen_create_dir(PPL_DIR, verbose) == -1)
    return -1;
    
  sprintf(dir, "%s/.%s", PPL_DIR, ARCH_DIR);
  if (gen_create_dir(dir, verbose) == -1)
    return -1;
  
  if (lgen_pvm_makefile(verbose) < 0)
    return -1;
    
  if (verbose)
    printf("PVM library files generated\n");
  
  return 0;
}

