/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library command line utility.                         */
/* PPL configuration file parser. The function ppl_parse() parse the       */
/* .ppl file of the routine and update the properties database             */
/* accordingly.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 13.01.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: lparser.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef LPARSER_H
#define _PARSER_H

#ifndef LPARSER_IMPL                        /* imported by a client module */

extern char *lparser_errors[];
extern int lparser_errno;
extern int lparser_act_line;

#else                             /* imported by the implementation module */

char *lparser_errors[] =                         /* possible parser errors */
{                                 /* must correspond to parser_errors_enum */
  "could not open library configuration file"
};

int lparser_errno;                               /* last error description */
int lparser_act_line;                    /* current line being parsed or 0 */

#endif

typedef enum              /* possible error codes returned by ppr_parser() */
{
  LERR_COULD_NOT_OPEN
} lparser_errors_enum;

/*-------------------------------------------------------------------------*/
/* ppl_parse:                                                              */
/* Parse the PPL configuration file (with extension .ppl) and update       */
/* the properties database accordingly.                                    */
/* If an error occurs then returns -1 and set the lparser_errno global     */
/* variable, returns 0 otherwise.                                          */
/*-------------------------------------------------------------------------*/
int ppl_parse (
  char *library_name,
  int verbose                            /* 1 if verbose mode, 0 otherwise */
);

#endif
