/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Library command line utility.                         */
/* Library properties manager. This is the repository of the properties    */
/* of a parallel library.                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 13.12.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: lprop.c                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel                                                   */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include <ppc/portable.h>

typedef struct _routine_name_type
{
  char name [256];
  struct _routine_name_type *next;
} routine_name_type;

static int nb_routines;               /* number of routines in the library */
static routine_name_type *first_routine;
static routine_name_type *last_routine_added;
static routine_name_type *next_routine_read;
static char lib_name [256];

/*-------------------------------------------------------------------------*/
/* lprop_init:                                                             */
/* Initializes the properties database.                                    */
/*-------------------------------------------------------------------------*/
void lprop_init (
  char *library_name
)
{
  nb_routines = 0;
  last_routine_added = NULL;
  strcpy(lib_name, library_name);
}

/*-------------------------------------------------------------------------*/
/* lprop_new_routine:                                                      */
/* Declares new routine for the library.                                   */
/*-------------------------------------------------------------------------*/
void lprop_new_routine (
  char *routine_name
)
{
  routine_name_type *ptr;

  if (last_routine_added == NULL)
    ptr = first_routine = malloc(sizeof(routine_name_type));
  else
    ptr = last_routine_added->next = malloc(sizeof(routine_name_type));

  strcpy(ptr->name, routine_name);
  ptr->next = NULL;

  last_routine_added = ptr;
  nb_routines++;
}

/*-------------------------------------------------------------------------*/
/* lprop_nb_routines:                                                      */
/* Returns the number of routines of the library.                          */
/*-------------------------------------------------------------------------*/
int lprop_nb_routines (void)
{
  return nb_routines;
}

/*-------------------------------------------------------------------------*/
/* lprop_get_routine_init:                                                 */
/* Must be called before any call to lprop_get_routine_next.               */
/*-------------------------------------------------------------------------*/
void lprop_get_routine_init (void)
{
  next_routine_read = first_routine;
}

/*-------------------------------------------------------------------------*/
/* lprop_get_routine_next:                                                 */
/* Return the next routine name, or NULL.                                  */
/*-------------------------------------------------------------------------*/
char *lprop_get_routine_next (void)
{
  routine_name_type *tmp;

  if (next_routine_read == NULL)
    return NULL;
    
  tmp = next_routine_read;
  next_routine_read = next_routine_read->next;

  return tmp->name;
}

/*-------------------------------------------------------------------------*/
/* lprop_print_report:                                                     */
/* Print library configuration report on standard output.                  */
/*-------------------------------------------------------------------------*/
void lprop_print_report (void)
{
  routine_name_type *ptr;
  
  printf("Parallel library configuration:\n");
  printf("  Name     : \"%s\"\n", lib_name);
  printf("  Routines : ");
  
  ptr = first_routine;
  while (ptr != NULL)
  {
    if (ptr != first_routine)
      printf("             ");
    printf("%s\n", ptr->name);
    ptr = ptr->next;
  }
}

/*-------------------------------------------------------------------------*/
/* lprop_lib_name:                                                         */
/* Return the library name.                                                */
/*-------------------------------------------------------------------------*/
char *lprop_lib_name (void)
{
  return lib_name;
}

