/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* PPC tools utilities.                                                    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.01.1995  Droux       Created                                         */
/* 20.01.1995  Droux       Use portability header file                     */
/* 03.02.1995  Droux       Added definitions for the Paragon               */
/* 20.04.1995  Droux       Added definitions for the IBM SP-2              */
/*-------------------------------------------------------------------------*/
/* filename: toolsutil.c                                                   */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define TOOS_UTIL_IMPL

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ppc/portable.h>
#include "toolsutil.h"

#define EXECV_FAILED 1234                     /* exit code if execv failed */

char *arch_names[] =                 /* strings corresponding to arch_enum */
{
  "unix",
  "pvm",
  "pactc",
  "paragon",
  "sp2"
};

char *output_arch_names[] =                   /* idem but for verbose mode */
{
  "UNIX",
  "PVM",
  "Transputer Networks",
  "Intel Paragon",
  "IBM SP-2"
};

/*-------------------------------------------------------------------------*/
/* run_cmd:                                                                */
/* Execute the specified command.                                          */
/* If the command exited with a non-null exit code, then exit with the     */
/* exit code.                                                              */
/*-------------------------------------------------------------------------*/
void run_cmd (
  char *path, 
  char *argv[],
  int verbose
)
{
  if (verbose)
  {
    int c;
    for (c = 0; argv[c] != NULL; c++)
      printf("%s ", argv[c]);
    printf("\n");
  }
  
  if (!fork())
  {                                                       /* child process */
    execv(path, argv);
    exit(EXECV_FAILED);
  }

  {
    #ifndef SUN4SOL2
    union wait statusp;
    #endif
    int exit_code;
    int status;
    
    #if defined(SUN4SOL2) || defined(PGON)
    wait(&status);
    #else
    wait(&statusp);
    status = statusp.w_status;
    #endif
    
    if ((status & 0177) == 0177)
      exit_code = -1;  /* stopped */
    else if (status && ((status & 0177) != 0177))
      exit_code = -1;  /* terminated by a signal */
    else
      exit_code = status >> 8;
      
    if (exit_code)
      exit(exit_code);
  }
}

