/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* Generators utilities.                                                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.01.1995  Droux       Created                                         */
/* 23.01.1995  Droux       Use portability header file                     */
/*-------------------------------------------------------------------------*/
/* filename: genutil.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <time.h>

#include <ppc/portable.h>
#include "genutil.h"

#define DEFAULT_MODE 07755                   /* default file creation mode */

/*-------------------------------------------------------------------------*/
/* gen_create_dir:                                                         */
/* Creates the specified directory if it does not yet exist.               */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int gen_create_dir (
  char *path,
  int verbose
)
{
  struct stat dir_stat;

  if(stat(path, &dir_stat) == -1)
  {
    if (verbose)
      printf("creating %s directory\n", path);
    if (mkdir(path, DEFAULT_MODE) == -1)
      return -1;
    if (verbose)
      printf("directory %s created\n", path);
  }
  
  return 0;
}

/*-------------------------------------------------------------------------*/
/* gen_get_user_name:                                                      */
/* Returns the user name:                                                  */
/*-------------------------------------------------------------------------*/
char *gen_get_user_name (void)
{
  static int user_name_init = 0;
  static char user_name [100];
  
  if (!user_name_init)
  {
    struct passwd *act_passwd;
    
    act_passwd = getpwuid(getuid());              /* get user's login name */
    strcpy(user_name, act_passwd->pw_name);
    user_name_init = 1;
  }
  
  return user_name;
}

/*-------------------------------------------------------------------------*/
/* gen_get_date:                                                           */
/* Returns the current date and time.                                      */
/*-------------------------------------------------------------------------*/
char *gen_get_date (void)
{
  static int date_str_init = 0;
  static char date_str [100];
  
  if (!date_str_init)
  {
    time_t act_time;
    
    time(&act_time);                         /* get string containing date */
    strcpy(date_str, ctime(&act_time));
    date_str[strlen(date_str)-1] = '\0';
    date_str_init = 1;
  }

  return date_str;
}

/*-------------------------------------------------------------------------*/
/* gen_split_path:                                                         */
/* Split a path into a directory and a file name.                          */
/* "file.c"                -> "", "file.c"                                 */
/* "file"                  -> "", "file"                                   */
/* "dir/file"              -> "dir/", "file"                               */
/* /dir/subdir/file.c      -> "/dir/subdir/", "file.c"                     */
/*-------------------------------------------------------------------------*/
void gen_split_path (
  char *path,
  char *dir,
  char *file
)
{
  int last_slash = 0;
  int idx;
  int last_point = 0;
  char newPath [256];
  char newName [256];
    
  for (idx = 0; path[idx] != '\0'; idx++)                  /* extract path */
    if (path[idx] == '/')
      last_slash = idx;
      
  if (!last_slash)
  {
    strcpy(newPath, "");
    strcpy(newName, path);
  }
  else
  {
    strcpy(newPath, path);
    newPath[last_slash+1] = '\0';
    for (idx = last_slash+1; path[idx] != '\0'; idx++)     /* extract name */
      newName[idx-(last_slash+1)] = path[idx];
    newName[idx-(last_slash+1)] = '\0';
  }
  
  
  strcpy(file, newName);
  strcpy(dir, newPath);
}

