/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* PPR configuration file parser. The function ppr_parse() parse the       */
/* .ppr file of the routine and update the properties database             */
/* accordingly.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 01.11.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: parser.h                                                      */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef PARSER_H
#define PARSER_H

#ifndef PARSER_IMPL                         /* imported by a client module */

extern char *parser_errors[];
extern int parser_errno;
extern int parser_act_line;

#else                             /* imported by the implementation module */

char *parser_errors[] =                          /* possible parser errors */
{                                 /* must correspond to parser_errors_enum */
  "could not open routine configuration file",
  "unknown property in routine configuration file",
  "unknown topology in routine configuration file",
  "routine configuration syntax error: parameter direction must be placed after a parameter type",
  "unknown parameter type in routine configuration file"
};

int parser_errno;                                /* last error description */
int parser_act_line;                     /* current line being parsed or 0 */

#endif

typedef enum              /* possible error codes returned by ppr_parser() */
{
  PERR_COULD_NOT_OPEN,
  PERR_UNKNOWN_PROP,
  PERR_UNKNOWN_TOPO,
  PERR_SYNTAX_DIR,
  PERR_UNKNOWN_PARAM
} parser_errors_enum;

/*-------------------------------------------------------------------------*/
/* ppr_parse:                                                              */
/* Parse the PPR configuration file (with extension .ppr) and update       */
/* the properties database accordingly.                                    */
/* If an error occurs then returns -1 and set the parser_errno global      */
/* variable, returns 0 otherwise.                                          */
/*-------------------------------------------------------------------------*/
int ppr_parse (
  char *routine_name,
  int verbose                            /* 1 if verbose mode, 0 otherwise */
);

#endif
