/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* Routine properties manager. This is the repository of the properties    */
/* of a parallel routine.                                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 01.11.1994  Droux       Created                                         */
/* 05.04.1995  Droux       Added STRING routine parameter type             */
/* 01.05.1995  Droux       Added complex type                              */
/* 03.05.1995  Droux       Added boolean type                              */
/*-------------------------------------------------------------------------*/
/* filename: prop.h                                                        */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef PROP_H
#define PROP_H

#ifndef PROP_IMPL                           /* imported by a client module */
                                            
extern char *topologies_str[];
extern char *parameter_types_str[];
extern char *directions_str[];
extern char *properties_str[];

#else                             /* imported by the implementation module */

char *topologies_str[] =                             /* topologies strings */
{                                      /* must correspond to topology_type */
  "FARM", 
  "RING", 
  "GRID"
};

char *parameter_types_str[] =                        /* parameters strings */
{                               /* must correspond to parameter_types_enum */
  "INT", 
  "SHORT", 
  "UNSIGNED_SHORT", 
  "LONG", 
  "UNSIGNED_LONG", 
  "FLOAT", 
  "DOUBLE", 
  "CHAR",
  "COMPLEX",
  "BOOLEAN",
  
  "INT_VECTOR", 
  "SHORT_VECTOR", 
  "UNSIGNED_SHORT_VECTOR", 
  "LONG_VECTOR",
  "UNSIGNED_LONG_VECTOR", 
  "FLOAT_VECTOR", 
  "DOUBLE_VECTOR", 
  "CHAR_VECTOR",
  "COMPLEX_VECTOR",
  "BOOLEAN_VECTOR",

  "INT_MATRIX", 
  "SHORT_MATRIX", 
  "UNSIGNED_SHORT_MATRIX", 
  "LONG_MATRIX",
  "UNSIGNED_LONG_MATRIX", 
  "FLOAT_MATRIX", 
  "DOUBLE_MATRIX", 
  "CHAR_MATRIX",
  "COMPLEX_MATRIX",
  "BOOLEAN_MATRIX",

  "STRING"
};

char *directions_str[] =                             /* directions strings */
{                                     /* must correspond to direction_type */
  "IN", 
  "OUT"      
};

char *properties_str[] =
{
  "topology", 
  "parameters", 
  "master_modules", 
  "slave_modules"
};

#endif

typedef enum                                        /* possible topologies */
{
  TOPO_FARM, 
  TOPO_RING, 
  TOPO_GRID,
  NB_TOPOLOGIES,
  TOPO_UNKNOWN
} topology_type;

typedef enum                                            /* parameter types */
{
  INT, 
  SHORT, 
  UNSIGNED_SHORT, 
  LONG, 
  UNSIGNED_LONG, 
  FLOAT, 
  DOUBLE, 
  CHAR,
  COMPLEX,
  BOOLEAN,
  
  INT_VECTOR, 
  SHORT_VECTOR, 
  UNSIGNED_SHORT_VECTOR, 
  LONG_VECTOR,
  UNSIGNED_LONG_VECTOR, 
  FLOAT_VECTOR, 
  DOUBLE_VECTOR, 
  CHAR_VECTOR,
  COMPLEX_VECTOR,
  BOOLEAN_VECTOR,
  
  INT_MATRIX, 
  SHORT_MATRIX, 
  UNSIGNED_SHORT_MATRIX, 
  LONG_MATRIX,
  UNSIGNED_LONG_MATRIX, 
  FLOAT_MATRIX, 
  DOUBLE_MATRIX, 
  CHAR_MATRIX,
  COMPLEX_MATRIX,
  BOOLEAN_MATRIX,
  
  STRING,
  NB_PARAMETER_TYPES
} parameter_types_enum;

typedef enum                                        /* parameter direction */
{
  IN, OUT,
  NB_DIRECTIONS
} direction_type;

typedef enum                                        /* possible properties */
{
  PROP_TOPOLOGY, 
  PROP_PARAMETERS, 
  PROP_MASTER_MODULES, 
  PROP_SLAVE_MODULES,
  NB_PROPERTIES
} properties_enum;

/*-------------------------------------------------------------------------*/
/* prop_init:                                                              */
/* Initializes the properties database.                                    */
/*-------------------------------------------------------------------------*/
void prop_init (
  char *routine_name
);

/*-------------------------------------------------------------------------*/
/* prop_set_topology:                                                      */
/* Set the routine topology.                                               */
/*-------------------------------------------------------------------------*/
void prop_set_topology (
  topology_type topo
);

/*-------------------------------------------------------------------------*/
/* prop_new_parameter:                                                     */
/* Add the specified parameter to the parameters list of the routine.      */
/*-------------------------------------------------------------------------*/
void prop_new_parameter (
  parameter_types_enum param
);

/*-------------------------------------------------------------------------*/
/* prop_param_direction:                                                   */
/* Set the direction of the last declared parameter.                       */
/* Returns -1 if no parameter has been previously defined.                 */
/*-------------------------------------------------------------------------*/
int prop_param_direction (
  direction_type dir
);

/*-------------------------------------------------------------------------*/
/* prop_new_master_module:                                                 */
/* Add the specified master module to the routine.                         */
/*-------------------------------------------------------------------------*/
void prop_new_master_module (
  char *module_name
);

/*-------------------------------------------------------------------------*/
/* prop_new_slave_module:                                                  */
/* Add the specified slave module to the routine.                          */
/*-------------------------------------------------------------------------*/
void prop_new_slave_module (
  char *module_name
);

/*-------------------------------------------------------------------------*/
/* prop_print_report:                                                      */
/* Print routine configuration report.                                     */
/*-------------------------------------------------------------------------*/
void prop_print_report (void);

/*-------------------------------------------------------------------------*/
/* prop_get_master_name:                                                   */
/* Returns the first name of the master files appearing                    */
/* in the list of master files.                                            */
/*-------------------------------------------------------------------------*/
char *prop_get_master_name (void);

/*-------------------------------------------------------------------------*/
/* prop_get_slave_name:                                                    */
/* Returns the first name of the slave files appearing                     */
/* in the list of slave files.                                             */
/*-------------------------------------------------------------------------*/
char *prop_get_slave_name (void);

/*-------------------------------------------------------------------------*/
/* prop_get_routine_name:                                                  */
/* Returns the name of the routine.                                        */
/*-------------------------------------------------------------------------*/
char *prop_get_routine_name (void);

/*-------------------------------------------------------------------------*/
/* prop_get_topology:                                                      */
/* Returns the topology of the routine.                                    */
/*-------------------------------------------------------------------------*/
topology_type prop_get_topology (void);

/*-------------------------------------------------------------------------*/
/* prop_get_param_init:                                                    */
/* Must be called before any call to prop_get_param:                       */
/*-------------------------------------------------------------------------*/
void prop_get_param_init (void);

/*-------------------------------------------------------------------------*/
/* prop_get_param:                                                         */
/* Returns the ANSI-C type of the next routine parameter.                  */
/* Returns NULL if no more parameters are available.                       */
/*-------------------------------------------------------------------------*/
char *prop_get_param (void);

/*-------------------------------------------------------------------------*/
/* prop_get_param_is_out:                                                  */
/* Returns 1 if the parameter returned by prop_get_param is an             */
/* output parameter.                                                       */ 
/*-------------------------------------------------------------------------*/
int prop_get_param_is_out (void);

/*-------------------------------------------------------------------------*/
/* prop_get_master_init:                                                   */
/* Must be called before any call to prop_get_master_next.                 */
/*-------------------------------------------------------------------------*/
void prop_get_master_init (void);

/*-------------------------------------------------------------------------*/
/* prop_get_master_next:                                                   */
/* Return the next master module name, or NULL.                            */
/*-------------------------------------------------------------------------*/
char *prop_get_master_next (void);

/*-------------------------------------------------------------------------*/
/* prop_get_slave_init:                                                    */
/* Must be called before any call to prop_get_slave_next.                  */
/*-------------------------------------------------------------------------*/
void prop_get_slave_init (void);

/*-------------------------------------------------------------------------*/
/* prop_get_slave_next:                                                    */
/* Return the next slave module name, or NULL.                             */
/*-------------------------------------------------------------------------*/
char *prop_get_slave_next (void);

#endif
