/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering School of Biel, Switzerland.                                */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Portable Parallel Routines command line utility.                        */
/* Routines generator. This module can generate the files needed to        */
/* build a parallel routine.                                               */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 06.01.1995  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: rgenerator.h                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.3, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1995 Engineering School of Biel-Bienne, Switzerland.      */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef RGENERATOR_H
#define RGENERATOR_H

typedef enum              /* the three possible methods to generate a file */
{
  GEN_DEFAULT,                          /* generate file with default name */
  GEN_STDOUT,                        /* redirect result to standard output */
  GEN_USE_NAME                        /* generate file with specified name */
} gen_method;

/*-------------------------------------------------------------------------*/
/* rgen_master:                                                            */
/* Generates the routine main master file.                                 */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_master (
  gen_method method,
  char *optional_name,
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_slave:                                                             */
/* Generates the routine main slave file.                                  */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_slave (
  gen_method method,
  char *optional_name,
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_user_interface:                                                    */
/* Generates the routine user interface.                                   */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_user_interface (
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_master_starter:                                                    */
/* Generates the master starter.                                           */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_master_starter (
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_slave_starter:                                                     */
/* Generates the slave starter.                                            */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_slave_starter (
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_master_header:                                                     */
/* Generates the master header file.                                       */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_master_header (
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_slave_header:                                                      */
/* Generates the slave header file.                                        */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_slave_header (
  int verbose
);

/*-------------------------------------------------------------------------*/
/* rgen_object_list:                                                       */
/* Generates in a special file the list of all object files that are part  */
/* of the routine and must be included in the library.                     */
/* Returns 0 on success, -1 otherwise.                                     */
/*-------------------------------------------------------------------------*/
int rgen_object_list (
  int verbose
);

#endif

