# ------------------------------------------------------------------------- #
#  Mathematics on Parallel and Distributed Systems                          #
#  Engineering College of Biel, Switzerland.                                #
#  Computer Science Dpt.                                                    #
#  Swiss National Science Foundation project 5003-034331.                   #
#  Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.     #
# ------------------------------------------------------------------------- #
#  Description:                                                             #
#  PPC: Portable Parallel C                                                 #
#  PPC libraries makefile. This file is given as parameter to make by       #
#  the makeit shell script.                                                 #
# ------------------------------------------------------------------------- #
#  History:                                                                 #
#  Date        By          Description                                      #
#  ----------  ----------- -----------------------------------------------  #
#  19.11.1993  Droux       Created                                          #
#  07.07.1994  Droux       New structure of source directory                #
#  15.07.1994  Droux       Added nextstep and loginpalette dependencies     #
#  03.08.1994  Droux       Added gethome dependency                         #
#  29.08.1994  Droux       NEXTSTEP configuration "standardized"            #
#  30.08.1994  Droux       added "base" dependency                          #
#  21.10.1994  Droux       added matrix utilities library                   #
#  28.10.1994  Droux       added routine manager                            #
#  01.11.1994  Droux       added lib dependency                             #
#  03.02.1995  Droux       create parlib directories automatically          #
#  03.02.1995  Droux       Added support for the Paragon                    #
#  23.02.1995  Droux       Added triple-fat "standard" client lib           #
#  28.03.1995  Droux       Added DDS                                        #
#  04.04.1995  Droux       Added default dependency                         #
#  29.04.1995  Droux       Added IBM SP-2 dependency                        #
# ------------------------------------------------------------------------- #
#  filename: ppc.mak                                                        #
#  Tested on :                                                              #
#    NEXTSTEP 3.2, Motorola                                                 #
#    NEXTSTEP 3.2, Intel                                                    # 
#    NEXTSTEP 3.3, Intel                                                    #
#    Linux 1.0.8, Intel                                                     #
#    SunOS 4.1.3, SPARC                                                     #
#    Solaris 2.3, SPARC                                                     #
#    Solaris 2.3, SPARC                                                     #
#    Solaris 2.4, SPARC                                                     #
#    Solaris 2.4, SPARC Multiprocessor                                      #
# ------------------------------------------------------------------------- #
#  Copyright (c) 1994, 1995 Engineering School of Biel-Bienne, Switzerland. #
#                                                                           #
#  This program is free software; you can redistribute it and/or modify it  #
#  under the terms of the GNU General Public License as published by the    #
#  Free Software Foundation; either version 2 of the License, or (at your   #
#  option) any later version.                                               #
#                                                                           #
#  This program is distributed in the hope that it will be useful, but      #
#  WITHOUT ANY WARRANTY; without even the implied warranty of               #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     #
#  See the GNU General Public License for more details.                     #
# ------------------------------------------------------------------------- #

LIBBASE=$(PPC_HOME)/lib
LIBDIR=$(LIBBASE)/$(HOST_ARCH)
PACTCDIR=$(PPC_HOME)/lib/PACTC
NEXTFATDIR=$(PPC_HOME)/lib/NEXTFAT
PGONDEFS=CC=icc OTHER_CFLAGS=-nx HOST_ARCH=PGON
SP2DEFS=SP=1 _BSD=43

default:
	@ cat INSTALL

base: $(LIBBASE) $(LIBDIR)
	@ echo "[making base tools]"
	@ cd tools/portable/ppccmd; make

all: base unix pvm pactc client nextstep lib

nextstep: loginpalette nxppccenter

lib: $(LIBBASE) $(LIBDIR)
	@ echo "[making parallel library environment]"
	@ cd parlib/pprcmd; make pprcmd
	@ cd parlib/pplcmd; make pplcmd

unix: $(LIBBASE) $(LIBDIR) base
	@ echo "[making UNIX implementation]"
	@ cd csp; make unix
	@ cd config; make unix
	@ cd cslib/server; make unix
	@ cd matrixutil; make unix
	@ cd parlib/config; make unix
	@ cd parlib/kernel; make unix
	@ cd parlib/dds; make unix
	@ ppc_mkdirs $(PPC_HOME)/parlib/unix/$(HOST_ARCH)

pvm: $(LIBBASE) $(LIBDIR) base
	@ echo "[making PVM implementation]"
	@ cd csp; make pvm
	@ cd config; make pvm
	@ cd cslib/server; make pvm
	@ cd cslib/pvm; make
	@ cd matrixutil; make pvm
	@ cd parlib/config; make pvm
	@ cd parlib/kernel; make pvm
	@ cd parlib/dds; make pvm
	@ ppc_mkdirs $(PPC_HOME)/parlib/pvm/$(HOST_ARCH)
	
sp2: $(LIBBASE) $(LIBDIR) base
	@ echo "[making IBM SP-2 implementation (using PVMe)]"
	@ cd csp; make pvm $(SP2DEFS)
	@ cd config; make sp2 $(SP2DEFS)
	@ cd cslib/server; make pvm $(SP2DEFS)
	@ cd cslib/pvm; make $(SP2DEFS)
	@ cd matrixutil; make pvm $(SP2DEFS)
	@ cd parlib/config; make pvm $(SP2DEFS)
	@ cd parlib/kernel; make pvm $(SP2DEFS)
	@ cd parlib/dds; make pvm $(SP2DEFS)
	@ ppc_mkdirs $(PPC_HOME)/parlib/pvm/$(HOST_ARCH)
	
pactc: $(LIBBASE) $(LIBDIR) $(PACTCDIR) base
	@ echo "[making transputer PACT-C implementation]"
	@ cd csp; make pactc
	@ cd config; make pactc
	@ cd cslib/server; make pactc
	@ cd cslib/pactc/pserver; make -f Makefile.$(HOST_ARCH)
	@ cd cslib/pactc/tranredir; make 
	@ cd matrixutil; make pactc
	@ cd parlib/config; make pactc
	@ cd parlib/kernel; make pactc
	@ cd parlib/dds; make pactc
	@ ppc_mkdirs $(PPC_HOME)/parlib/pactc/$(HOST_ARCH)
	
paragon: $(LIBBASE) $(LIBDIR) base
	@ echo "[making Intel Paragon implementation]"
	@ cd config; make paragon
	@ cd config; make paragon $(PGONDEFS)
	@ cd csp; make paragon
	@ cd csp; make paragon $(PGONDEFS)
	@ cd matrixutil; make paragon
	@ cd matrixutil; make paragon $(PGONDEFS)
	@ cd parlib/config; make paragonc
	@ cd parlib/config; make paragon $(PGONDEFS)
	@ cd cslib/server; make paragon $(PGONDEFS)
	@ cd cslib/server; make paragon
	@ cd parlib/kernel; make paragon $(PGONDEFS)
	@ cd parlib/dds; make paragon $(PGONDEFS)

client: $(LIBBASE) $(LIBDIR) base
	@ echo "[making CS-LIB client library]"
	@ cd cslib/client; make
	
loginpalette: $(LIBBASE) $(NEXTFATDIR) base
	@ echo "[making NEXTSTEP CS-LIB extensions]"
	@ cd cslib/LoginPalette; makeit
	@ cd $(NEXTFATDIR); rm -f libcscli.a; ln -s libNXPPCKit.a libcscli.a

nxppccenter: $(LIBBASE) $(NEXTFATDIR) base
	@ echo "[making NEXTSTEP PPCCenter]"
	@ cd tools/nextstep/PPCCenter; makeit

clean:
	@ rm -f $(PPC_HOME)/lib/$(HOST_ARCH)/*
	@ rm -f $(PPC_HOME)/lib/PACTC/*
	@ rm -rf $(PPC_HOME)/lib/NEXTFAT/*
	@ rm -rf cslib/LoginPalette/i386_m68k_obj
	@ rm -rf cslib/LoginPalette/LoginPalette.palette

$(LIBBASE):
	mkdir $(LIBBASE)

$(PACTCDIR):
	mkdir $(PACTCDIR)

$(NEXTFATDIR):
	mkdir $(NEXTFATDIR)
	
$(LIBDIR): 
	mkdir $(LIBDIR)
